/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __BASE_BASE_H
#define __BASE_BASE_H

#include "base.types.h"
#include "base.assert.h"
#include "base.new.h"

#include <memory.h>

namespace magnum {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void ConstructElements(T* pElements, int nCount) {
	// first do bit-wise zero initialization
	// memset((void*)pElements, 0, nCount * sizeof(T));

	// then call the constructor(s)
	for (; nCount--; pElements++)
		::new((void*)pElements) T;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void DestructElements(T* pElements, int nCount) {
	// call the destructor(s)
	for (; nCount--; pElements++)
		pElements->~T();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void ConstructElement(T* pElement) {
	// first do bit-wise zero initialization
	// memset((void*)pElements, 0, nCount * sizeof(T));

	::new((void*)pElement) T;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void DestructElement(T* pElement) {
	pElement->~T();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void CopyElements(T* pDest, const T* pSrc, int nCount) {
	// default is element-copy using assignment
	while (nCount--)
		*pDest++ = *pSrc++;
}

};

#endif
