/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __BASE_NUMBER_H
#define __BASE_NUMBER_H

#include "object.h"

namespace magnum { namespace types {
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Number : public Object {
DeclareClass(Number, Object);
public:
	virtual unsigned char	byteValue() const = 0;
	virtual short			shortValue() const = 0;
	virtual int				intValue() const = 0;
	virtual long			longValue() const = 0;
	virtual float			floatValue() const = 0;
	virtual double			doubleValue() const = 0;
	virtual bool			boolValue() const = 0;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Byte : public Number {
DeclareClass(Byte, Number);
private:
	unsigned char	m_Value;

public:
	inline			Byte();
	inline			Byte(unsigned char );
	inline			Byte(const Byte& b);

	unsigned char	byteValue() const;
	short			shortValue() const;
	int				intValue() const;
	long			longValue() const;
	float			floatValue() const;
	double			doubleValue() const;
	bool			boolValue() const;

	String			toString() const;

	static Byte	getByte(const String& , unsigned int radix = 10);
	static Byte	parseByte(const String& , unsigned int radix = 10);

	inline bool operator==(const Byte& str) const;
	inline bool operator!=(const Byte& str) const;
	inline bool operator>(const Byte& str) const;
	inline bool operator<(const Byte& str) const;
	inline bool operator>=(const Byte& str) const;
	inline bool operator<=(const Byte& str) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Short : public Number {
DeclareClass(Short, Number);
private:
	short			m_Value;

public:
	inline			Short();
	inline			Short(short );
	inline			Short(const Short& );

	unsigned char	byteValue() const;
	short			shortValue() const;
	int				intValue() const;
	long			longValue() const;
	float			floatValue() const;
	double			doubleValue() const;
	bool			boolValue() const;

	String			toString() const;

	static Short	getShort(const String& , unsigned int radix = 10);
	static Short	parseShort(const String& , unsigned int radix = 10);

	inline bool operator==(const Short& str) const;
	inline bool operator!=(const Short& str) const;
	inline bool operator>(const Short& str) const;
	inline bool operator<(const Short& str) const;
	inline bool operator>=(const Short& str) const;
	inline bool operator<=(const Short& str) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Long : public Number {
DeclareClass(Long, Number);
private:
	long			m_Value;

public:
	inline			Long();
	inline			Long(long );
	inline			Long(const Long& );

	unsigned char	byteValue() const;
	short			shortValue() const;
	int				intValue() const;
	long			longValue() const;
	float			floatValue() const;
	double			doubleValue() const;
	bool			boolValue() const;

	String			toString() const;

	static Long	getLong(const String& , unsigned int radix = 10);
	static Long	parseLong(const String& , unsigned int radix = 10);

	inline bool operator==(const Long& str) const;
	inline bool operator!=(const Long& str) const;
	inline bool operator>(const Long& str) const;
	inline bool operator<(const Long& str) const;
	inline bool operator>=(const Long& str) const;
	inline bool operator<=(const Long& str) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Integer : public Number {
DeclareClass(Integer, Number);
private:
	int				m_Value;

public:
	inline			Integer();
	inline			Integer(int );
	inline			Integer(const Integer& );

	unsigned char	byteValue() const;
	short			shortValue() const;
	int				intValue() const;
	long			longValue() const;
	float			floatValue() const;
	double			doubleValue() const;
	bool			boolValue() const;

	String			toString() const;

	static Integer	getInt(const String& , unsigned int radix = 10);
	static Integer	parseInt(const String& , unsigned int radix = 10);

	inline bool operator==(const Integer& str) const;
	inline bool operator!=(const Integer& str) const;
	inline bool operator>(const Integer& str) const;
	inline bool operator<(const Integer& str) const;
	inline bool operator>=(const Integer& str) const;
	inline bool operator<=(const Integer& str) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Double : public Number {
DeclareClass(Double, Number);
private:
	double			m_Value;

public:
	inline			Double();
	inline			Double(double );
	inline			Double(const Double& );

	unsigned char	byteValue() const;
	short			shortValue() const;
	int				intValue() const;
	long			longValue() const;
	float			floatValue() const;
	double			doubleValue() const;
	bool			boolValue() const;

	String			toString() const;

	static Double	getDouble(const String& , unsigned int radix = 10);
	static Double	parseDouble(const String& , unsigned int radix = 10);

	inline bool operator==(const Double& str) const;
	inline bool operator!=(const Double& str) const;
	inline bool operator>(const Double& str) const;
	inline bool operator<(const Double& str) const;
	inline bool operator>=(const Double& str) const;
	inline bool operator<=(const Double& str) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Float : public Number {
DeclareClass(Float, Number);
private:
	float			m_Value;

public:
	inline			Float();
	inline			Float(float );
	inline			Float(const Float& );

	unsigned char	byteValue() const;
	short			shortValue() const;
	int				intValue() const;
	long			longValue() const;
	float			floatValue() const;
	double			doubleValue() const;
	bool			boolValue() const;

	String			toString() const;

	static Float	getFloat(const String& , unsigned int radix = 10);
	static Float	parseFloat(const String& , unsigned int radix = 10);

	inline bool operator==(const Float& str) const;
	inline bool operator!=(const Float& str) const;
	inline bool operator>(const Float& str) const;
	inline bool operator<(const Float& str) const;
	inline bool operator>=(const Float& str) const;
	inline bool operator<=(const Float& str) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Bool : public Number {
DeclareClass(Bool, Number);
private:
	bool			m_Value;

public:
	inline			Bool();
	inline			Bool(bool );
	inline			Bool(const Bool& );

	unsigned char	byteValue() const;
	short			shortValue() const;
	int				intValue() const;
	long			longValue() const;
	float			floatValue() const;
	double			doubleValue() const;
	bool			boolValue() const;

	String			toString() const;

	static Bool	getBool(const String& , unsigned int radix = 10);
	static Bool	parseBool(const String& , unsigned int radix = 10);

	inline bool operator==(const Bool& str) const;
	inline bool operator!=(const Bool& str) const;
	inline bool operator>(const Bool& str) const;
	inline bool operator<(const Bool& str) const;
	inline bool operator>=(const Bool& str) const;
	inline bool operator<=(const Bool& str) const;
};

}; };
#endif
