/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_ALLOCATOR_INL
#define __CONTAINER_ALLOCATOR_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> Allocator<T>::Allocator() {
	m_Size = 64;
	m_First = (Pool*)(void *)new byte[sizeof(Pool) + 63*Tsize];
	m_Last = m_First;
	m_First->m_Next = NULL;
	m_Free = (void *)&m_First->m_Data[0];

	void* data = m_Free;
	void* next;
	for (unsigned int i = 0; i < 63; i++) {
		next = (void*)((byte *)data + Tsize);
		*(void **)data = next;
		data = next;
	}
	*(void **)data = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> Allocator<T>::~Allocator() {

}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T*	Allocator<T>::alloc() {
	if (m_Free == NULL)
		allocate();
	T* ret = (T*)m_Free;
	m_Free = *(void **)m_Free;
	return ret;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Allocator<T>::free(T* data) {
	*(void **)data = (void *)m_Free;
	m_Free = (void **)data;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void Allocator<T>::allocate() {
	Pool* pool = (Pool*)(void *)new byte[sizeof(Pool) + (m_Size-1)*Tsize];
	m_Last->m_Next = pool;
	m_Last = pool;

	void* data = &pool->m_Data[0];
	void* next;
	for (unsigned int i = 0; i < m_Size-1; i++) {
		next = (void*)((byte *)data + Tsize);
		*(void **)data = next;
		data = next;
	}

	*(void **)data = m_Free;
	m_Free = &m_Last->m_Data[0];

	m_Size += m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void Allocator<T>::deallocate();


}; };

#endif
