/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_ARRAY_H
#define __CONTAINER_ARRAY_H

#include "object.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

#pragma warning(disable:4284)


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Array : public Object {
DeclareClass(Array, Object);
private:
	unsigned int	m_Size;
	unsigned int	m_MaxSize;
	T*				m_Array;

public:
	class Iterator : public IteratorBase<Iterator> {
	friend class Array<T>;
	public:
		inline Iterator() {
		};
		inline operator T*() {
			return m_Item;
		};
		inline operator const T*() const {
			return m_Item;
		};
		inline T& operator*() const {
			return *m_Item;
		};
		inline T* operator->() const {
			return m_Item;
		};
		inline Iterator& operator++() {
			ASSERT(m_Item != NULL);
			m_Item++;
			return *this;
		};
		inline Iterator operator++(int ) {
			ASSERT(m_Item != NULL);
			T* item = m_Item;
			m_Item++;
			return Iterator(item);
		};
		inline Iterator& operator+=(int n) {
			m_Item += n;
			return *this;
		};
		inline Iterator operator+(int n) const {
			return Iterator(m_Item + n);
		};
		inline Iterator& operator--() {
			ASSERT(m_Item != NULL);
			m_Item--;
			return *this;
		};
		inline Iterator	operator--(int ) {
			ASSERT(m_Item != NULL);
			T* item = m_Item;
			m_Item--;
			return Iterator(item);
		};
		inline Iterator& operator-=(int n) {
			m_Item -= n;
			return *this;
		};
		inline Iterator operator-(int n) const {
			return Iterator(m_Item - n);
		};
		inline bool	operator==(const Iterator& iter) const {
			return m_Item == iter.m_Item;
		};
		inline bool	operator<(const Iterator& iter) const {
			return m_Item < iter.m_Item;
		};
		inline bool	operator<=(const Iterator& iter) const {
			return m_Item <= iter.m_Item;
		};
		inline bool	operator>(const Iterator& iter) const {
			return m_Item > iter.m_Item;
		};
		inline bool	operator>=(const Iterator& iter) const {
			return m_Item >= iter.m_Item;
		};

	private:
		inline Iterator(T* item) {
			m_Item = item;
		};

		T*	m_Item;
	};

	
public:
	inline Array();
	inline Array(const Array& );
	inline ~Array();

	void				clear();
	inline unsigned int getSize() const;
	void				setSize(unsigned int n);
	inline bool			isEmpty() const;
	
	
	inline unsigned int	getCapacity() const;
	void				setCapacity(unsigned int n);

	inline const T&		get(unsigned int n) const;
	inline void			set(unsigned int n, const T& elem);

	inline const T*		getArray() const;
	inline T*			getArray();

	inline Iterator		begin() const;
	inline Iterator		at(unsigned int ) const;
	inline Iterator		end() const;

	inline unsigned int	insert(const T& item);
	void				insert(unsigned int n, const T& item);
	void				remove(unsigned int n);
	void				remove(const Iterator& iter);
	bool				contains(const T& item) const;

	void				insert(unsigned int n, const Array& );
	void				append(const Array& );
	void				copy(const Array& );

	void				shrink();

	inline T&			operator [](unsigned int nIndex );
	inline const T&		operator [](unsigned int nIndex ) const;

	const Array&		operator=(const Array& );
};

}; };
#endif
