/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_BINARYTREE_H
#define __CONTAINER_BINARYTREE_H


#include "base.types.h"
#include "object.h"
#include "base.assert.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

template<class Key, class Item> class BinaryTreeEnumerator;


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item> class BinaryTree : public Object {
DeclareClass(BinaryTree, Object);
friend class BinaryTreeEnumerator<Key, Item>;
private:
	typedef struct Node {
		Item	m_Item;
		Key		m_Key;
		Node*	m_LSon;
		Node*	m_RSon;
		Node*	m_Father;
	} Node;


	Node *m_Root;

public:
	class Iterator : public IteratorBase<Iterator> {
	friend class BinaryTree<Key, Item>;
		public:
			inline Iterator() {
				m_Node = NULL;
			};
			inline Item&		operator*() const {
				return m_Node->m_Item;
			};
			inline Item*		operator->() const {
				return &(m_Node->m_Item);
			};
			inline Iterator& operator++() {
				ASSERT(m_Node != NULL);
				// Try to get the next value
				if (m_Node->m_RSon != NULL) {
					m_Node = m_Node->m_RSon;
					while (m_Node->m_LSon != NULL)
						m_Node = m_Node->m_LSon;
					return *this;
				}

				for (;;) {
					Node* father = m_Node->m_Father;
					if (father == NULL) {
						m_Node = NULL;
						return *this;
					}
					if (father->m_LSon == m_Node) {
						m_Node = father;
						return *this;
					}
					m_Node = father;
				}

				return *this;
			};
			inline Iterator operator++(int ) {
				ASSERT(m_Node != NULL);
				Iterator iter = *this;
				operator++();
				return iter;
			};
			inline Iterator& operator--() {
				ASSERT(m_Node != NULL);

				// Try to get the next value
				if (m_Node->m_LSon != NULL) {
					m_Node = m_Node->m_LSon;
					while (m_Node->m_RSon != NULL)
						m_Node = m_Node->m_RSon;
					return *this;
				}

				for (;;) {
					Node* father = m_Node->m_Father;
					if (father == NULL) {
						m_Node = NULL;
						return *this;
					}
					if (father->m_RSon == m_Node) {
						m_Node = father;
						return *this;
					}
					m_Node = father;
				}

				return *this;
			};
			inline Iterator	operator--(int ) {
				ASSERT(m_Node != NULL);
				Iterator iter = *this;
				operator--();
				return iter;
			};
			inline bool		operator==(const Iterator& iter) {
				return m_Node == iter.m_Node;
			};
			inline bool		operator<(const Iterator& iter) {
				return m_Node < iter.m_Node;
			};

		Node*	m_Node;
	};


public:
	inline BinaryTree();
	inline ~BinaryTree();

	inline void clear();

	inline bool insert(const Item& item,const Key& key);
	inline bool remove(const Key& key);
	inline bool remove(const Item& item);

	inline bool get(const Key& key, Item& item) const;
	inline bool get(const Item& item, Key& key) const;

	inline Item* get(const Key& key) const;
	inline const Key* get(const Item& item) const;

	inline bool contains(const Item& item) const;
	inline bool contains(const Key& key) const;

	inline Item getRoot() const;

	inline bool isEmpty() const;

	inline Iterator	begin() const;
	inline Iterator	end() const;

private:
	inline void removeNode(Node *node, Node **father);
};


}; };

#endif
