/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_CHAIN_H
#define __CONTAINER_CHAIN_H

#include "object.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

#pragma warning(push)
#pragma warning(disable:4284)

template<class T> class Chain;
template<class T> class Chainable;
template<class T> class ChainIterator;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Chainable {
friend class Chain<T>;
friend class ChainIterator<T>;
private:
	Chain<T>*		m_Chain;
	Chainable<T>*	m_Next;
	Chainable<T>*	m_Prev;

public:
	inline Chainable();
	inline ~Chainable();

	inline Chain<T>*	getChain() const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class ChainIterator : public IteratorBase<ChainIterator> {
friend class Chain<T>;
public:
	inline ChainIterator() {
	};
	inline operator T*() {
		return (T*)m_Item;
	};
	inline operator const T*() const {
		return (T*)m_Item;
	};
	inline T& operator*() const {
		return *(T*)m_Item;
	};
	inline T* operator->() const {
		return (T*)m_Item;
	};
	inline ChainIterator& operator++() {
		ASSERT(m_Item != NULL);
		m_Item = m_Item->m_Next;
		return *this;
	};
	inline ChainIterator operator++(int ) {
		ASSERT(m_Item != NULL);
		Chainable<T>* item = m_Item;
		m_Item = m_Item->m_Next;
		return ChainIterator(item);
	};
	inline ChainIterator& operator--() {
		ASSERT(m_Item != NULL);
		m_Item = m_Item->m_Prev;
		return *this;
	};
	inline ChainIterator	operator--(int ) {
		ASSERT(m_Item != NULL);
		Chainable<T>* item = m_Item;
		m_Item = m_Item->m_Prev;
		return ChainIterator(item);
	};
	inline bool	operator==(const ChainIterator& iter) {
		return m_Item == iter.m_Item;
	};
	inline bool	operator<(const ChainIterator& iter) {
		return m_Item < iter.m_Item;
	};

private:
	inline ChainIterator(Chainable<T>* item) {
		m_Item = item;
	};

	Chainable<T>*	m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Chain : public Object {
DeclareClass(Chain, Object);
friend class Chainable<T>;
private:
	Chainable<T>		m_Root;
	unsigned int		m_Size;

public:
	typedef ChainIterator<T>	Iterator;

public:
	inline	Chain();
			Chain(Chain& );
	inline	~Chain();

	inline bool		contains(const T* item) const;
	Iterator		insert(T* item);
	Iterator		insertFirst(T* item);
	void			remove(T* item);
	inline void		remove(const Iterator& iter);
	void			clear();

	inline bool		isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator	begin() const;
	inline Iterator at(const T* item) const;
	inline Iterator	end() const;

	const Chain&	operator=(Chain &);
};



#pragma warning(pop)

}; };
#endif
