/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_ITERATORBASE_INL
#define __CONTAINER_ITERATORBASE_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool IteratorBase<T>::operator!=(const T& iter) const {
	return !((*(T *)this) == iter);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool IteratorBase<T>::operator>=(const T& iter) const {
	return (((*(T *)this) > iter) || ((*(T *)this) == iter));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool IteratorBase<T>::operator<=(const T& iter) const {
	return !((*(T *)this) > iter);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool IteratorBase<T>::operator<(const T& iter) const {
	return ((!((*(T *)this) == iter)) && !((*(T *)this) > iter));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T& IteratorBase<T>::operator+=(int n) {
	if (n>0) {
		for(;n>0;n--)
			(*(T *)this)++;
	} else if (n<0) {
		for(;n<0;n++)
			(*(T *)this)--;
	}

	return *(T *)this;
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T& IteratorBase<T>::operator-=(int n) {
	if (n>0) {
		for(;n>0;n--)
			(*(T *)this)--;
	} else if (n<0) {
		for(;n<0;n++)
			(*(T *)this)++;
	}

	return *(T *)this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T IteratorBase<T>::operator+(int n) const {
	T iter = *this;
	if (n>0) {
		for(;n>0;n--)
			iter++;
	} else if (n<0) {
		for(;n<0;n++)
			iter--;
	}

	return iter;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T IteratorBase<T>::operator-(int n) const {
	T iter = *this;
	if (n>0) {
		for(;n>0;n--)
			iter--;
	} else if (n<0) {
		for(;n<0;n++)
			iter++;
	}

	return iter;
}

}; };
#endif
