/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_LIST_H
#define __CONTAINER_LIST_H

#include "object.h"
#include "container.iterator.h"
#include "container.allocator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

#pragma warning(push)
#pragma warning(disable:4284)

template<class T> class List;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class List : public Object {
DeclareClass(List, Object);
private:
	class Item {
	public:
		T			m_Item;
		Item*		m_Next;
		Item*		m_Prev;
	public:
		Item() {};
		Item(const T& i) : m_Item(i) {};
	};

	Allocator<Item>	m_Allocator;
	Item*			m_First;
	Item			m_Terminator;
	unsigned int	m_Size;

public:
	class Iterator : public IteratorBase<Iterator> {
	friend class List<T>;
	public:
		inline Iterator() {
		};
		inline operator T*() {
			return &m_Item->m_Item;
		};
		inline operator const T*() const {
			return &m_Item->m_Item;
		};
		inline T& operator*() const {
			return m_Item->m_Item;
		};
		inline T* operator->() const {
			return &m_Item->m_Item;
		};
		inline Iterator& operator++() {
			ASSERT(m_Item != NULL);
			m_Item = m_Item->m_Next;
			return *this;
		};
		inline Iterator operator++(int ) {
			ASSERT(m_Item != NULL);
			Item* item = m_Item;
			m_Item = m_Item->m_Next;
			return Iterator(item);
		};
		inline Iterator& operator--() {
			ASSERT(m_Item != NULL);
			m_Item = m_Item->m_Prev;
			return *this;
		};
		inline Iterator	operator--(int ) {
			ASSERT(m_Item != NULL);
			Item* item = m_Item;
			m_Item = m_Item->m_Prev;
			return Iterator(item);
		};
		inline bool operator==(const Iterator& iter) const {
			return m_Item == iter.m_Item;
		};
		inline bool operator!=(const Iterator& iter) const {
			return m_Item != iter.m_Item;
		};
		inline bool operator<(const Iterator& iter) const {
			return m_Item < iter.m_Item;
		};
		inline bool operator>(const Iterator& iter) const {
			return m_Item > iter.m_Item;
		};
		inline bool operator<=(const Iterator& iter) const {
			return m_Item <= iter.m_Item;
		};
		inline bool operator>=(const Iterator& iter) const {
			return m_Item >= iter.m_Item;
		};

	private:
		inline Iterator(Item* item) {
			m_Item = item;
		};

		Item*	m_Item;
	};


public:
	inline List();
	inline List(const List& );
	inline ~List();

	bool			contains(const T& item) const;
	Iterator		insert(const T& item);
	Iterator		insertFirst(const T& item);
	bool			remove(const T& item);
	void			remove(const Iterator& iter);
	void			clear();

	inline bool		isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator	begin() const;
	inline Iterator at(const T& ) const;
	inline Iterator	end() const;

	const List&		operator=(const List &);
};


#pragma warning(pop)

}; };
#endif
