/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_MULTISET_INL
#define __CONTAINER_MULTISET_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------------
Description:
	Inserts a new item into the binary tree. If there is already an item with 
	the same key, then the item will be substituted by the new item

Parameters:
	item - item to insert
	key - key to associate with the item

Remarks:
	Only one item can be associated with one key, although more than one
	key could be associated with one item. When you insert an item with
	an already existing key, the previosu item associtaed with the key 
	will be substituted.
----------------------------------------------------------------------------------*/
template<class T> 
typename Set<T>::Iterator Multiset<T>::insert(const T& item) {
	Node *node = m_Root;
	Node *newNode;
	
	if (node == NULL) {
		m_Root = new(m_Allocator) Node;
		m_Root->m_LSon =
		m_Root->m_RSon =
		m_Root->m_Father = NULL;
		m_Root->m_Item = item;
		m_Root->m_Rand = rand();
		m_Size++;
		return Iterator(m_Root);
	}	

	for (;;) {
		if (node->m_Item < item) {
			if (node->m_RSon == NULL) {
				newNode = node->m_RSon = new(m_Allocator) Node;
				newNode->m_Father = node;
				newNode->m_Item = item;
				newNode->m_LSon =
				newNode->m_RSon = NULL;
				newNode->m_Rand = rand();
				repairUp(newNode);
				m_Size++;
				return Iterator(newNode);
			} else {
				node = node->m_RSon;
			}
		} else {
			if (node->m_LSon == NULL) {
				newNode = node->m_LSon = new(m_Allocator) Node;
				newNode->m_Father = node;
				newNode->m_Item = item;
				newNode->m_LSon =
				newNode->m_RSon = NULL;
				newNode->m_Rand = rand();
				repairUp(newNode);
				m_Size++;
				return Iterator(newNode);
			} else {
				node = node->m_LSon;
			}
		}
	}
}

}; };

#endif
