/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_QUEUE_H
#define __CONTAINER_QUEUE_H

#include "object.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;


#pragma warning(push)
#pragma warning(disable:4284)

template<class T> class Queue;
template<class T> class Queueable;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Queue : public Object {
DeclareClass(Queue, Object);
private:
	Queueable<T>*		m_Head;
	Queueable<T>*		m_Tail;
	unsigned int		m_Size;

public:
	class Iterator : public IteratorBase<Iterator> {
	public:
		inline Iterator() {
		};
		inline operator T*() {
			return (T*)m_Item;
		};
		inline operator const T*() const {
			return (T*)m_Item;
		};
		inline T& operator*() const {
			return *(T*)m_Item;
		};
		inline T* operator->() const {
			return (T*)m_Item;
		};
		inline Iterator& operator++() {
			ASSERT(m_Item != NULL);
			m_Item = m_Item->m_Next;
			return *this;
		};
		inline Iterator operator++(int ) {
			ASSERT(m_Item != NULL);
			Queueable<T>* item = m_Item;
			m_Item = m_Item->m_Next;
			return Iterator(item);
		};
		inline Iterator& operator--() {
			ASSERT(m_Item != NULL);
			m_Item = m_Item->m_Prev;
			return *this;
		};
		inline Iterator	operator--(int ) {
			ASSERT(m_Item != NULL);
			Queueable<T>* item = m_Item;
			m_Item = m_Item->m_Prev;
			return Iterator(item);
		};
		inline bool	operator==(const Iterator& iter) {
			return m_Item == iter.m_Item;
		};
		inline bool	operator<(const Iterator& iter) {
			return m_Item < iter.m_Item;
		};

	private:
		inline Iterator(Queueable<T>* item) {
			m_Item = item;
		};

		Queueable<T>*	m_Item;
	};


public:
	inline Queue();
	inline ~Queue();

	inline bool		contains(const T* item) const;
	void			insert(T* item);
	void			insertHead(T* item);
	void			remove(T* item);
	void			clear();

	inline T*		removeHead();
	inline T*		peekHead() const;
	inline T*		removeTail();
	inline T*		peekTail() const;

	inline bool		isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator	begin() const;
	inline Iterator	end() const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Queueable {
friend class Queue<T>;
friend class Queue<T>::Iterator;
private:
	Queue<T>*		m_Queue;
	Queueable<T>*	m_Next;
	Queueable<T>*	m_Prev;

public:
	inline Queueable();
	inline ~Queueable();
};


#pragma warning(pop)

}; };


#endif
