/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_SET_H
#define __CONTAINER_SET_H

#include "base.types.h"
#include "object.h"
#include "base.assert.h"
#include "container.iterator.h"
#include "container.allocator.h"


namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

template<class T> class Multiset;

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> class Set : public Object {
DeclareClass(Set, Object);
friend class Multiset<T>;
private:
	struct Node {
		T				m_Item;
		unsigned int	m_Rand;
		Node*			m_LSon;
		Node*			m_RSon;
		Node*			m_Father;
	};


	Node*				m_Root;
	unsigned int		m_Size;
	unsigned int		m_Rand;
	Allocator<Node>		m_Allocator;


public:
	class Iterator : public IteratorBase<Iterator> {
	friend class Set<T>;
	friend class Multiset<T>;
	public:
		inline Iterator() {
		};
		inline operator T*() {
			return &m_Node->m_Item;
		};
		inline operator const T*() const {
			return &m_Node->m_Item;
		};
		inline T& operator*() const {
			return m_Node->m_Item;
		};
		inline T* operator->() const {
			return &m_Node->m_Item;
		};
		inline Iterator& operator++() {
			ASSERT(m_Node != NULL);
			// Try to get the next value
			if (m_Node->m_RSon != NULL) {
				m_Node = m_Node->m_RSon;
				while (m_Node->m_LSon != NULL)
					m_Node = m_Node->m_LSon;
				return *this;
			}

			for (;;) {
				Node* father = m_Node->m_Father;
				if (father == NULL) {
					m_Node = NULL;
					return *this;
				}
				if (father->m_LSon == m_Node) {
					m_Node = father;
					return *this;
				}
				m_Node = father;
			}

			return *this;
		};
		inline Iterator operator++(int ) {
			ASSERT(m_Node != NULL);
			Node* node = m_Node;
			operator++();
			return Iterator(node);
		};
		inline Iterator& operator--() {
			ASSERT(m_Node != NULL);

			// Try to get the next value
			if (m_Node->m_LSon != NULL) {
				m_Node = m_Node->m_LSon;
				while (m_Node->m_RSon != NULL)
					m_Node = m_Node->m_RSon;
				return *this;
			}

			for (;;) {
				Node* father = m_Node->m_Father;
				if (father == NULL) {
					m_Node = NULL;
					return *this;
				}
				if (father->m_RSon == m_Node) {
					m_Node = father;
					return *this;
				}
				m_Node = father;
			}

			return *this;
		};
		inline Iterator	operator--(int ) {
			ASSERT(m_Node != NULL);
			Node* node = m_Node;
			operator--();
			return Iterator(node);
		};
			inline bool	operator==(const Iterator& iter) const {
			return m_Node == iter.m_Node;
		};
		inline bool	operator<(const Iterator& iter) const {
			return m_Node < iter.m_Node;
		};

	private:
		inline Iterator(Node* item) {
			m_Node = item;
		};

		Node*	m_Node;
	};


public:
	inline Set();
	inline ~Set();

	inline bool		contains(const T& item) const;
	Iterator		insert(const T& item);
	void			remove(const T& item);
	void			remove(const Iterator& item);
	void			clear();

	inline bool		isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator	begin() const;
	inline Iterator at(const T& item) const;
	inline Iterator	end() const;

	bool			operator==(const Set& ) const;
	bool			operator!=(const Set& ) const;

private:
	void			removeNode(Node *node, Node **father);
	void			repairUp(Node *node);
	inline unsigned int rand();
};


}; };
#endif
