/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __CONTAINER_STACK_H
#define __CONTAINER_STACK_H

#include "object.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template <class T> class Stack : public Object {
DeclareClass(Stack, Object);
private:
	T*       	 	m_Stack;
	unsigned int	m_Position;
	unsigned int	m_Size;

public:
	class Iterator : public IteratorBase<Iterator> {
	friend class Stack<T>;
	public:
		inline Iterator() {
		};
		inline operator T*() {
			return m_Item;
		};
		inline operator const T*() const {
			return m_Item;
		};
		inline T& operator*() const {
			return *m_Item;
		};
		inline T* operator->() const {
			return m_Item;
		};
		inline Iterator& operator++() {
			ASSERT(m_Item != NULL);
			m_Item++;
			return *this;
		};
		inline Iterator operator++(int ) {
			ASSERT(m_Item != NULL);
			T* item = m_Item;
			m_Item++;
			return Iterator(item);
		};
		inline Iterator& operator+=(int n) {
			m_Item += n;
			return iter;
		};
		inline Iterator& operator--() {
			ASSERT(m_Item != NULL);
			m_Item--;
			return *this;
		};
		inline Iterator	operator--(int ) {
			ASSERT(m_Item != NULL);
			T* item = m_Item;
			m_Item--;
			return Iterator(item);
		};
		inline Iterator& operator-=(int n) {
			m_Item -= n;
			return iter;
		};
		inline Iterator operator+(int n) const {
			return Iterator(m_Item + n);
		};
		inline Iterator operator-(int n) const {
			return Iterator(m_Item - n);
		};
		inline bool		operator==(const Iterator& iter) const {
			return m_Item == iter.m_Item;
		};
		inline bool		operator<(const Iterator& iter) const {
			return m_Item < iter.m_Item;
		};

	private:
		inline Iterator(T* item) {
			m_Item = item;
		};

		T*	m_Item;
	};


public:
	explicit Stack(int size = 64);
	~Stack();

	inline void    		push(const T& );
	inline T 			pop(void);
	inline T&			peek(void);
	inline const T&		peek(void) const;
						
	inline void    		clear(void);
	inline bool    		isEmpty(void) const;
	inline bool    		isFull(void) const;
	inline unsigned int	getSize() const;

	inline Iterator		begin() const;
	inline Iterator		at(unsigned int ) const;
	inline Iterator		end() const;

	inline T&			operator[](unsigned int );
	inline const T&		operator[](unsigned int ) const;
};


}; };
#endif
