/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_BOX_H
#define __GEOMETRY_BOX_H

#include "math.vector.h"
#include "geometry.body.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;

template<class T, int dim> class Sphere;
template<class T, int dim> class AlignedBox;
template<class T, int dim> class Cube;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Box : public Body<T,dim> {
private:
	Vector<T,dim>		m_Origin;
	Vector<T,dim>		m_Axes[dim];
	T					m_Axes2[dim];

public:
	inline void			setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void			setAxes(const Vector<T,dim>* );
	inline const Vector<T,dim>* getAxes() const;
	inline void			setAxis(unsigned int ,const Vector<T,dim>& );
	inline const Vector<T,dim>& getAxis(unsigned int ) const;

	inline bool			isEmpty() const; 
	inline void			empty(); 

	inline bool			isInside(const Vector<T,dim>& ) const; 
	T					getDistance(const Vector<T,dim>& ) const;
	
	bool				intersect(const Sphere<T,dim>& ) const;
	bool				intersect(const AlignedBox<T,dim>& ) const;
	bool				intersect(const Box<T,dim>& ) const;
	bool				intersect(const Cube<T,dim>& ) const;

	void				bound(const Vector<T,dim>* ,unsigned int n);
	void				bound(const Vector<T,dim>* ,unsigned int stride,unsigned int n);

	void				getVertices(Vector<T,dim>* ) const;
};


template<class T> class Box2 : public Box<T,2> {};
template<class T> class Box3 : public Box<T,3> {};
template<class T> class Box4 : public Box<T,4> {};


}; };

#endif
