/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_CAPSULE_H
#define __GEOMETRY_CAPSULE_H

#include "geometry.segment.h"
#include "geometry.body.h"

namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Capsule : public Body<T,dim> {
private:
	Segment<T,dim>	m_Segment;
	T				m_Radius;

public:
	inline Capsule();
	inline Capsule(const Vector<T,dim>& ,const Vector<T,dim>& ,const T& );

	inline void		setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void		setDirection(const Vector<T,dim>& );
	inline const Vector<T,dim>& getDirection() const;

	inline void		setRadius(const T& );
	inline T		getRadius() const;

	inline void		setLength(const T& );
	inline T		getLength() const;

	inline bool		isEmpty() const;

	inline bool		isInside(const Vector<T,dim>& ) const; 
	inline T		getDistance(const Vector<T,dim>& ) const;

	void			bound(const Vector<T,dim>* ,unsigned int n);
	void			bound(const Vector<T,dim>* ,unsigned int stride,unsigned int n);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Capsule2 : public Capsule<T,2> {
public:
	inline Capsule2() {};
	inline Capsule2(const Capsule<T,2>& c) : Capsule(2) {};
	inline Capsule2(const Vector<T,2>& a, const Vector<T,2>& b, const T& r) : Capsule(a,b,r) {};
};

template<class T> class Capsule3 : public Capsule<T,3> {
public:
	inline Capsule3() {};
	inline Capsule3(const Capsule<T,3>& c) : Capsule(3) {};
	inline Capsule3(const Vector<T,3>& a, const Vector<T,3>& b, const T& r) : Capsule(a,b,r) {};
};

template<class T> class Capsule4 : public Capsule<T,4> {
public:
	inline Capsule4() {};
	inline Capsule4(const Capsule<T,4>& c) : Capsule(4) {};
	inline Capsule4(const Vector<T,4>& a, const Vector<T,4>& b, const T& r) : Capsule(a,b,r) {};
};

}; };

#endif
