/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_FRUSTUM_H
#define __GEOMETRY_FRUSTUM_H

#include "math.vector.h"
#include "math.vector3.h"
#include "geometry.body.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::math::Vector3;

template<class T, int dim> class Sphere;
template<class T, int dim> class Box;
template<class T, int dim> class AlignedBox;
template<class T, int dim> class Cube;
template<class T, int dim> class Bounds;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Frustum : public Body<T,dim> {
private:
	Vector<T,dim>		m_Origin;
	Vector<T,dim>		m_Axes[dim-1];
	Vector<T,dim>		m_Direction;
	T					m_Near;
	T					m_Far;

public:
	inline void			setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void			setAxes(const Vector<T,dim>* );
	inline const Vector<T,dim>* getAxes() const;
	inline void			setAxis(unsigned int ,const Vector<T,dim>& );
	inline const Vector<T,dim>& getAxis(unsigned int ) const;
	
	inline void			setDirection(const Vector<T,dim>& );
	inline const Vector<T,dim>& getDirection() const;
	
	inline void			setNear(const T& );
	inline const T&		getNear() const;

	inline void			setFar(const T& );
	inline const T&		getFar() const;

	inline bool			isEmpty() const;
						
	inline bool			isInside(const Vector<T,dim>& ) const; 
	inline T			getDistance(const Vector<T,dim>& ) const;

	inline bool			intersect(const Sphere<T,dim>& ) const;
	inline bool			intersect(const Cube<T,dim>& ) const;
	inline bool			intersect(const Box<T,dim>& ) const;
	inline bool			intersect(const AlignedBox<T,dim>& ) const;
	inline bool			intersect(const Bounds<T,dim>& ) const;
	
	inline Containment	cull(const Cube<T,dim>& ) const;
	inline Containment	cull(const Bounds<T,dim>& ) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Frustum3 : public Frustum<T,3> {
};

}; };

#endif
