/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_FRUSTUM_INL
#define __GEOMETRY_FRUSTUM_INL

namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Frustum<T,dim>::setOrigin(const Vector<T,dim>& v) {
	m_Origin = v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Frustum<T,dim>::getOrigin() const {
	return m_Origin;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void	Frustum<T,dim>::setAxes(const Vector<T,dim>* e) {
	for (unsigned int i = 0; i < dim; i++) {
		m_Axes[i] = e[i];
		m_Axes2[i] = m_Axes[i] * m_Axes[i];
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>* Frustum<T,dim>::getAxes() const {
	return m_Axes;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void	Frustum<T,dim>::setAxis(unsigned int i, const Vector<T,dim>& a) {
	ASSERT(i < dim);
	m_Axes[i] = a;
	m_Axes2[i] = m_Axes[i] * m_Axes[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>& Frustum<T,dim>::getAxis(unsigned int i) const {
	ASSERT(i < dim);
	return m_Axes[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Frustum<T,dim>::setDirection(const Vector<T,dim>& v) {
	m_Direction = v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Frustum<T,dim>::getDirection() const {
	return m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Frustum<T,dim>::setNear(const T& near) {
	m_Near = near;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const T& Frustum<T,dim>::getNear() const {
	return m_Near;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Frustum<T,dim>::setFar(const T& near) {
	m_Far = near;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const T& Frustum<T,dim>::getFar() const {
	return m_Far;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Frustum<T,dim>::isEmpty() const {
	ASSERT(false);
	return false;
}
					

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Frustum<T,dim>::isInside(const Vector<T,dim>& ) const {
	ASSERT(false);
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Frustum<T,dim>::getDistance(const Vector<T,dim>& ) const {
	ASSERT(false);
	return T(0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Frustum<T,dim>::intersect(const Sphere<T,dim>& ) const {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Frustum<T,dim>::intersect(const Cube<T,dim>& ) const {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Frustum<T,dim>::intersect(const AlignedBox<T,dim>& ) const {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Frustum<T,dim>::intersect(const Box<T,dim>& ) const {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Frustum<T,dim>::intersect(const Bounds<T,dim>& ) const {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Containment Frustum<T,dim>::cull(const Cube<T,dim>& ) const {
	return Subset;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Containment Frustum<T,dim>::cull(const Bounds<T,dim>& ) const {
	return Subset;
}

}; };

#endif
