/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_RAY_H
#define __GEOMETRY_RAY_H

#include "math.vector.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Ray {
private:
	Vector<T,dim>	m_Origin;
	Vector<T,dim>	m_Direction;

public:
	inline Ray();
	inline Ray(const Vector<T,dim>& ,const Vector<T,dim>& );

	inline void		setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void		setDirection(const Vector<T,dim>& );
	inline const Vector<T,dim>& getDirection() const;

	inline T		getDistance2(const Vector<T,dim>& ) const;
	inline T		getDistance(const Vector<T,dim>& ) const;
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Ray2 : public Ray<T,2> {
public:
	inline Ray2() {};
	inline Ray2(const Ray<T,2>& l) : Ray(l) {};
	inline Ray2(const Vector<T,2>& o,const Vector<T,2>& d) : Ray(o,d) {};
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Ray3 : public Ray<T,3> {
public:
	inline Ray3() {};
	inline Ray3(const Ray<T,3>& l) : Ray(l) {};
	inline Ray3(const Vector<T,3>& o,const Vector<T,3>& d) : Ray(o,d) {};
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Ray4 : public Ray<T,4> {
public:
	inline Ray4() {};
	inline Ray4(const Ray<T,4>& l) : Ray(l) {};
	inline Ray4(const Vector<T,4>& o,const Vector<T,4>& d) : Ray(o,d) {};
};

}; };

#endif
