/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_SEGMENT_INL
#define __GEOMETRY_SEGMENT_INL

namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline Segment<T,dim>::Segment() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline Segment<T,dim>::Segment(const Vector<T,dim>& o,const Vector<T,dim>& d) {
	m_Origin = o;
	m_Direction = d;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void	Segment<T,dim>::setOrigin(const Vector<T,dim>& o) {
	m_Origin = o;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>& Segment<T,dim>::getOrigin() const {
	return m_Origin;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void	Segment<T,dim>::setDirection(const Vector<T,dim>& d) {
	m_Direction = d;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>& Segment<T,dim>::getDirection() const {
	return m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Segment<T,dim>::setLength(const T& l) {
	m_Direction *= l / m_Direction.length();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Segment<T,dim>::getLength() const {
	return m_Direction.length();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Segment<T,dim>::getDistance2(const Vector<T,dim>& v) const {
    Vector<T,dim> diff = v - m_Origin;
    T t = diff * m_Direction;

    if ( t <= T(0.0)) {
        t = T(0.0); 
    } else {
		T len = m_Direction.length2();
		if (t >= len) {
	        diff -= m_Direction;
		} else {
	        t /= len;
			diff -= t * m_Direction;
		}
    }

    return diff.length2();	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Segment<T,dim>::getDistance(const Vector<T,dim>& v) const {
    Vector<T,dim> diff = v - m_Origin;
    T t = diff * m_Direction;

    if ( t <= T(0.0)) {
        t = T(0.0); 
    } else {
		T len = m_Direction.length2();
		if (t >= len) {
	        diff -= m_Direction;
		} else {
	        t /= len;
			diff -= t * m_Direction;
		}
    }

    return diff.length();
}

}; };

#endif
