/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GEOMETRY_TRIANGLEMESH_INL
#define __GEOMETRY_TRIANGLEMESH_INL


namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline TriangleMesh<T,dim>::TriangleMesh() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline TriangleMesh<T,dim>::~TriangleMesh() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
void	TriangleMesh<T,dim>::append(const TriangleMesh* tmesh) {
	unsigned int add = m_Vertices.getSize();
	unsigned int count = tmesh->getIndices().getSize();
	const unsigned int* indices = tmesh->getIndices().getArray();
	unsigned int oldcount = m_Indices.getSize();
	unsigned int* dst;
	m_Vertices.append(tmesh->getVertices());
	m_Indices.setSize(oldcount + count);
	dst = m_Indices.getArray() + oldcount;

	unsigned int i;
	for (i = 0; i < count; i++) {
		*dst = *indices + add;
		dst++;
		indices++;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
void TriangleMesh<T,dim>::optimize(T eps) {
	magnum::container::Map<vec, Item> gridmap;
	magnum::container::Map<vec, Item>::Iterator iter;
	Array<unsigned int> indexmap;
	Item item;
	vec vector, gvector;
	unsigned int index = 0;
	unsigned int i, j;

	indexmap.setSize(m_Vertices.getSize());

	// Sort all vertices into the grid
	for (i = 0; i < m_Vertices.getSize(); i++) {
		// Get vector
		vector = m_Vertices[i];

		// Put it onto the grid
		if (eps > (T)0) {
			for (j = 0; j < dim; j++)
				gvector[j] = (T)(int)(vector[j]/eps);
		} else {
			gvector = vector;
		}

		// Search gridded vector
		iter = gridmap.at(gvector);
		if (iter != gridmap.end()) {
			iter->m_Vector += vector;
			iter->m_Count++;
			indexmap[i] = iter->m_Index;
		} else {
			item.m_Vector = vector;
			item.m_Count = 1;
			item.m_Index = index;
			gridmap.insert(gvector, item);
			indexmap[i] = index;
			index++;
		}
	}

	// Create new vertex array
	m_Vertices.clear();
	m_Vertices.setSize(gridmap.getSize());
	for (iter = gridmap.begin(); iter != gridmap.end(); iter++) {
		m_Vertices[iter->m_Index] = iter->m_Vector / (T)iter->m_Count;
	}

	// Substitute all indices in index array
	for (i = 0; i < m_Indices.getSize(); i++) {
		m_Indices[i] = indexmap[m_Indices[i]];
	} 
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::setTriangle(unsigned int n, unsigned int i1,unsigned int i2,unsigned int i3) {
	n *= 3;
	m_Indices.setSize(n+3);
	m_Indices[n] = i1;
	m_Indices[n+1] = i2;
	m_Indices[n+2] = i3;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::setTriangle(unsigned int n, const Vector<T,dim>& v1,const Vector<T,dim>& v2,const Vector<T,dim>& v3) {
	ASSERT(false);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::setTriangle(unsigned int n, const Triangle<T,dim>& t) {
	setTriangle(n, t.a, t.b, t.c);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
void	TriangleMesh<T,dim>::clear() {
	m_Indices.clear();
	m_Vertices.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::setIndices(const unsigned int* src,unsigned int n) {
	m_Indices.setSize(n);
	memcpy(m_Indices.getArray(), src, n*sizeof(unsigned int));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::setIndices(unsigned int n) {
	m_Indices.setSize(n);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::getIndices(unsigned int* dst,unsigned int n) {
	memcpy(dst, m_Indices.getArray(), min(n, m_Indices.getSize())*sizeof(unsigned int));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::setVertices(const Vector<T,dim>* src,unsigned int n) {
	m_Vertices.setSize(n);
	memcpy(m_Vertices.getArray(), src, n*sizeof(Vector<T,dim>));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::setVertices(unsigned int n) {
	m_Vertices.setSize(n);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::getVertices(Vector<T,dim>* dst,unsigned int n) {
	memcpy(dst, m_Vertices.getArray(), min(n, m_Vertices.getSize())*sizeof(Vector<T,dim>));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline unsigned int TriangleMesh<T,dim>::getSize() {
	return m_Indices.getSize() / 3;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Array<unsigned int>& TriangleMesh<T,dim>::getIndices() const {
	return m_Indices;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Array< Vector<T,dim> >& TriangleMesh<T,dim>::getVertices() const {
	return m_Vertices;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<unsigned int,3> TriangleMesh<T,dim>::getIndices(unsigned int i) const {
	Vector<unsigned int, 3> vec;
	i *= 3;
	vec[0] = m_Indices[i];
	vec[1] = m_Indices[i+1];
	vec[2] = m_Indices[i+2];
	return vec;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Triangle<T,dim> TriangleMesh<T,dim>::getVertices(unsigned int i) const {
	i *= 3;
	return Triangle<T,dim>(m_Vertices[i], m_Vertices[i+1], m_Vertices[i+2]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> TriangleMesh<T,dim>::getVertex(unsigned int i) const {
	return m_Vertices[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void TriangleMesh<T,dim>::setVertex(unsigned int i, const Vector<T,dim>& v) {
	m_Vertices[i] = v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Triangle<T,dim> TriangleMesh<T,dim>::operator[](unsigned int i) const{
	i *= 3;
	return Triangle<T,dim>(m_Vertices[i], m_Vertices[i+1], m_Vertices[i+2]);
}

}; };

#endif
