/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS_IMAGE_H
#define __GRAPHICS_IMAGE_H

#include "graphics.rectangle.h"
#include "graphics.dimension.h"
#include "graphics.palette.h"

namespace magnum { namespace graphics {

class Context;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Image : public Object {
DeclareClass(Image, Object);
public:
	enum Type {
		Unknown		=	0x00000000,
		RGBA8		= 	0x00200001,
		RGB5		=	0x00100002,
		R5G6B5		=	0x00100003,
		RGB8		=	0x00180004,
		RGB12		=	0x00240005,
		Index8		=	0x00080006,
		BGR8		=	0x00180007,
		BGRA8		=	0x00200008,
		RGB16		=	0x00100009,
		RGBA16		=	0x0040000a,
		BGR16		=	0x0010000b,
		BGRA16		=	0x0040000c,
		GR8			=	0x0008000d,
		A8			=	0x0008000d,
		GR16		=	0x0010000e,
		A16			=	0x0010000e,
		GRA8		=	0x0010000f,
		GRA16		=	0x00200010,
		DUDV8		=	0x00100011,
		DUDV16		=	0x00200012,
		RGBA16f		=	0x10400013,
		RGB16f		=	0x10300014,
		BGRA16f		=	0x10400015,
		BGR16f		=	0x10300016,
		GR16f		=	0x10100017,
		A16f		=	0x10100017,
		GRA16f		=	0x10200018,
		RGBA32f		=	0x30800019,
		RGB32f		=	0x3060001a,
		BGRA32f		=	0x3080001b,
		BGR32f		=	0x3060001c,
		GR32f		=	0x3020001d,
		A32f		=	0x3020001e,
		GRA32f		=	0x3040001f,
		DXT1		=	0x40000040,
		DXT3		=	0x40000041,
		DXT5		=	0x40000042,
	};

private:
	Palette<byte>*			m_Palette;
	Image*					m_Next;
	Dimension<int>			m_Size;
	int						m_Linesize;
	Type					m_Type;
	void*					m_Pixels;
	bool					m_Locked;

public:
	Image();
	Image(Type type, const Dimension<int>& size);
	Image(Type type, int w, int h);
	Image(const Image* );
	Image(const Image& );
    ~Image();

	inline Dimension<int>	getSize() const;
	inline int				getWidth() const;
	inline int				getHeight() const;
	inline Type				getType() const;
	inline int				getLinesize() const;
	inline int				getPixelsize() const;

	inline bool				isIndexed() const;
	inline bool				isPacked() const;

	inline void				attachNext(Image* img);
	inline void				detachNext();
	inline Image*			getNext() const;

	bool					copyPixels(const Image* src);
	bool					copyPalette(const Image* src);

	inline bool				getPalette(Palette<byte>* pal) const;
	inline bool				setPalette(const Palette<byte>& pal);

	inline void*			lock();
	inline void				unlock();
	inline bool				isLocked() const;

protected:
	int						calcLinesize();
	inline void				setLinesize(int ls);
	inline void				setType(Type t);

	virtual void*			onLock();
	virtual void			onUnlock();
	virtual bool			onGetPalette(Palette<byte>* ) const;
	virtual bool			onSetPalette(const Palette<byte>&);
};


}; };

#endif
