/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_CAMERA_INL
#define __GRAPHICS3D_CAMERA_INL


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Camera::Camera() {
	m_Matrix.identity();
	m_NearPlane = 10.0f;
	m_FarPlane = 100.0f;
	m_ViewPlane = 20.0f;
	m_ViewWindow = Rectangle<float>(-10,-10,20,20);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Camera::setViewPlane(float f) {
	m_ViewPlane = f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Camera::getViewPlane() const {
	return m_ViewPlane;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setViewWindow(const Rectangle<float>& r) {
	m_ViewWindow = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setNearPlane(float f) {
	m_NearPlane = f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Camera::getNearPlane() const {
	return m_NearPlane;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setFarPlane(float f) {
	m_FarPlane = f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Camera::getFarPlane() const {
	return m_FarPlane;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setTransform(const Matrix4<float>& r) {
	m_Matrix = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setTransform(const Matrix3<float>& r) {
	for (int i = 0; i < 3; i++)
		for (int j = 0; j < 3; j++)
			m_Matrix[i][j] = r[i][j];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setPosition(const Vector3<float>& p) {
	m_Matrix[0][3] = p.x;
	m_Matrix[1][3] = p.y;
	m_Matrix[2][3] = p.z;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Camera::select() const {
	onSelect();
}

}; };

#endif
