/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_COMPILER_H
#define __GRAPHICS3D_COMPILER_H

#include "graphics3d.texture.h"
#include "graphics3d.mesh.h"
#include "graphics3d.vertexformat.h"
#include "container.array.h"
#include "container.set.h"
#include "object.pointer.h"
#include "system.assembler.h"


namespace magnum {namespace graphics3d {
using magnum::container::Set;
using magnum::object::Pointer;
using magnum::system::Program;
using magnum::system::Assembler;

class VertexBuffer;
class IndexBuffer;
class Shader;
class Mode;
class Material;
class PixelProgram;
class VertexProgram;
class RenderStream;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Compiler : public Object {
DeclareClass(Compiler, Object);
private:
	typedef Pointer<Resource> ResPtr;

	Set<ResPtr>				m_Resources;
	Assembler				m_Assembler;
							
	const Texture*			m_CurrentTexture[Texture::MaxTextures];						
    const Material*			m_CurrentMaterial;
    const Shader*			m_CurrentShader;
    const Mode*				m_CurrentMode;
    const VertexBuffer*		m_CurrentVertexBuffer[Attributes::MaxAttributes];
    const IndexBuffer*		m_CurrentIndexBuffer;
	const PixelProgram*		m_CurrentPixelProgram;
	const VertexProgram*	m_CurrentVertexProgram;
    
public:
    Compiler();
    ~Compiler();

    void				selectVertexBuffer(const VertexBuffer* , unsigned int attribs = 0xffffffff);
    void				selectIndexBuffer(const IndexBuffer* );
    void				selectMaterial(const Material* );
    void				selectTexture(const Texture* ,unsigned int );
    void				selectShader(const Shader* );
    void				selectMode(const Mode* );
	void				selectAppearance(const Appearance* );

    void				deselectMaterial();
    void				deselectTexture(unsigned int );
    void				deselectShader();
    void				deselectMode();
	void				deselectAppearance();

    void				renderVertices(Mesh::Primitive, unsigned int start, unsigned int size);
    void				renderIndices(Mesh::Primitive, unsigned int start, unsigned int size);
    
	void				clear();
	void				finish();
	RenderStream*		compile();
};


}; };

#endif
