/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_FRAMEBUFFER_H
#define __GRAPHICS3D_FRAMEBUFFER_H

#include "object.h"
#include "graphics.colour.h"
#include "graphics.rectangle.h"
#include "graphics.dimension.h"
#include "graphics.point.h"
#include "graphics.image.h"
#include "graphics3d.opengl.h"


namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::graphics::Colour;
using magnum::graphics::Rectangle;
using magnum::graphics::Dimension;
using magnum::graphics::Point;
using magnum::graphics::Image;

class RenderTarget;
class Context;
class Texture1D;
class Texture2D;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class FrameBuffer : public Object {
DeclareClass(FrameBuffer, Object);
friend RenderTarget;
friend Context;
private:
	RenderTarget*		m_RenderTarget;
	Dimension<int>		m_Size;

#if defined(__FORWINDOWS__)
	HDC					m_DeviceContext;
	HWND				m_WindowHandle;
#endif

public:
	enum ClearFlag {
		ClearColour = GL_COLOR_BUFFER_BIT,
		ClearDepth = GL_DEPTH_BUFFER_BIT,
		ClearStencil = GL_STENCIL_BUFFER_BIT,
		ClearAccum = GL_ACCUM_BUFFER_BIT
	};


private:
	FrameBuffer();

public:
	void					setSize(const Dimension<int>& );
	inline Dimension<int>	getSize() const;

	// Clearing stuff
	inline void				setClearColour(const Colour<float>& );
	inline Colour<float>	getClearColour() const;
	inline void				setClearAccum(const Colour<float>& );
	inline Colour<float>	getClearAccum() const;
	inline void				setClearDepth(float );
	inline float			getClearDepth() const;
	inline void				setClearStencil(unsigned int );
	inline unsigned int		getClearStencil() const;
	inline void				clear(unsigned int c);
					
	// Viewport
	inline void				setViewport(const Rectangle<float>& );
	inline Rectangle<float> getViewport() const;

	// Clipping / Scissoring
	inline void				setClipRectangle(const Rectangle<int>& );
	inline Rectangle<int>	getClipRectangle() const;
	inline void				setClipping(bool );
	inline bool				getClipping() const;

	// RenderTarget stuff
	inline RenderTarget*	getRenderTarget() const;

	// Image copying
	void					copyTo(Texture1D* ,unsigned int level, const Point<int>& ,const Rectangle<int>& ) const;
	void					copyTo(Texture2D* ,unsigned int level, const Point<int>& ,const Rectangle<int>& ) const;
	void					copyTo(Image* img, const Point<int>& ,const Rectangle<int>& ) const;
	void					copyFrom(const Image* img, const Point<int>& ,const Rectangle<int>& );
};

}; };

#endif
