/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_FRAMEBUFFER_INL
#define __GRAPHICS3D_FRAMEBUFFER_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<int> FrameBuffer::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FrameBuffer::setClearColour(const Colour<float>& c) {
	glClearColor(c.R, c.G, c.B, c.A);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Colour<float> FrameBuffer::getClearColour() const {
	GLfloat color[4];
	glGetFloatv(GL_COLOR_CLEAR_VALUE, color);
	return Colour<float>(color[0], color[1], color[2], color[3]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FrameBuffer::setClearAccum(const Colour<float>& c) {
	glClearAccum(c.R, c.G, c.B, c.A);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Colour<float> FrameBuffer::getClearAccum() const {
	GLfloat color[4];
	glGetFloatv(GL_ACCUM_CLEAR_VALUE, color);
	return Colour<float>(color[0], color[1], color[2], color[3]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FrameBuffer::setClearDepth(float d) {
	glClearDepth(d);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float FrameBuffer::getClearDepth() const {
	GLfloat val;
	glGetFloatv(GL_DEPTH_CLEAR_VALUE, &val);
	return val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FrameBuffer::setClearStencil(unsigned int s) {
	glClearStencil(s);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int FrameBuffer::getClearStencil() const {
	GLint val;
	glGetIntegerv(GL_STENCIL_CLEAR_VALUE, &val);
	return val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FrameBuffer::clear(unsigned int c) {
	glClear(c);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FrameBuffer::setViewport(const Rectangle<float>& r) {
	glViewport((int)r.getX(), m_Size.getHeight() - ((int)r.getY() + (int)r.getHeight()), (int)r.getWidth(), (int)r.getHeight());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Rectangle<float> FrameBuffer::getViewport() const {
	GLfloat f[4];
	glGetFloatv(GL_VIEWPORT, f);
	return Rectangle<float>(f[0], (float)m_Size.getHeight() - (f[1] + f[3]), f[2], f[3]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	FrameBuffer::setClipRectangle(const Rectangle<int>& r) {
	glScissor(r.getX(), m_Size.getHeight() - (r.getY() + r.getHeight()), r.getWidth(), r.getHeight());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Rectangle<int> FrameBuffer::getClipRectangle() const {
	GLint f[4];
	glGetIntegerv(GL_SCISSOR_BOX, f);
	return Rectangle<int>(f[0], m_Size.getHeight() - (f[1] + f[3]), f[2], f[3]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	FrameBuffer::setClipping(bool s) {
	if (s)
		glEnable(GL_SCISSOR_TEST);
	else
		glDisable(GL_SCISSOR_TEST);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FrameBuffer::getClipping() const {
	GLboolean b;
	glGetBooleanv(GL_SCISSOR_TEST, &b);
	return (b == GL_TRUE);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline RenderTarget* FrameBuffer::getRenderTarget() const {
	return m_RenderTarget;
}


}; };

#endif
