/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_INDEXBUFFER_H
#define __GRAPHICS3D_INDEXBUFFER_H

#include "system.assembler.h"
#include "graphics3d.resource.h"

namespace magnum { namespace graphics3d {
using magnum::system::Assembler;
class IndexMesh;
class Material;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class IndexBuffer : public Resource {
DeclareClass(IndexBuffer, Resource);
private:
	int				m_Type;
	unsigned int	m_Size;
	void*			m_Buffer;
	bool			m_Locked;
	unsigned int	m_OglUsage;
	unsigned int	m_OglBuffer;
	unsigned int	m_OglType;

	static const int g_OglAcces[];
	static const int g_OglUsage[];
	static const int g_OglType[5];

protected:
	IndexBuffer();

public:
	enum LockFlag {
		ReadOnly = 1,
		WriteOnly = 2,
		ReadWrite = 3,
		Discard = 4
	};

	enum CreateFlag {
		Static = 1,
		Dynamic = 2,
		Stream = 3,
		System = 4
	};

	enum Type {
		Index8 = 1,
		Index16 = 2,
		Index32 = 4
	};
	
	static IndexBuffer* create(Type t, unsigned int count, CreateFlag = Dynamic);

	~IndexBuffer();

	inline void*		lock(LockFlag = ReadWrite);
	inline void			unlock();
	inline bool			isLocked() const;

	inline void			select() const;
	void				compileSelect(Assembler& ) const;

	inline void			setBuffer(unsigned int offset, const void* data, unsigned int size);
	inline void			getBuffer(unsigned int offset, void* data, unsigned int size) const;

	inline unsigned int	getSize() const;
	inline Type			getType() const;

private:
	inline void*		bindBuffer() const;
};

}; };


#endif
