/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MESHMERGER_H
#define __GRAPHICS3D_MESHMERGER_H

#include "graphics3d.mesh.h"
#include "graphics3d.indexbuffer.h"
#include "graphics3d.vertexbuffer.h"
#include "container.list.h"
#include "container.tuple.h"
#include "container.map.h"


namespace magnum { namespace graphics3d {
using magnum::container::List;
using magnum::container::Tuple;
using magnum::container::Map;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MeshMerger : public Object {
DeclareClass(MeshMerger, Object);
private:
	typedef Tuple<VertexBuffer*, VertexBuffer::MaxBuffers> VbTuple;

	struct VbInfo {
		VertexFormat		m_Format;
		VertexBuffer*		m_Buffer;
		unsigned int		m_Size;
	};							
								
	struct VbMapping {			
		VbInfo*				m_Target;
		unsigned int		m_Start;
		unsigned int		m_Count;
	};

	struct IbInfo {
		IndexBuffer::Type	m_Format;
		IndexBuffer*		m_Buffer;
		unsigned int		m_Size;
	};

	struct IbMapping {
		IbInfo*				m_Target;
		unsigned int		m_Start;
		unsigned int		m_Count;
		unsigned int		m_Shift;
	};

	Map<VbTuple, VbMapping>	m_VbMapping;
	Map<IndexBuffer*, IbMapping> m_IbMapping;
	List<VbInfo>			m_VbInfo;
	List<IbInfo>			m_IbInfo;
	List<Mesh*>				m_Meshes;

public:
	struct IndexMapping {
		IndexBuffer*		m_Buffer;
		unsigned int		m_Start;
		unsigned int		m_Size;
		unsigned int		m_Shift;
	};

	struct VertexMapping {
		VertexBuffer*		m_Buffer;
		unsigned int		m_Start;
		unsigned int		m_Size;
	};

public:
	MeshMerger();
	~MeshMerger();

	void				clear();
	void				add(Mesh* );
	void				add(VertexBuffer* );
	void				add(VertexBuffer* ,IndexBuffer* );
	void				add(const Tuple<VertexBuffer*, VertexBuffer::MaxBuffers>& );
	void				add(const Tuple<VertexBuffer*, VertexBuffer::MaxBuffers>& ,IndexBuffer* );
	void				finish();

	IndexMapping		getMapping(const IndexBuffer* );
	VertexMapping		getMapping(const Tuple<VertexBuffer*, VertexBuffer::MaxBuffers>& );
	VertexMapping		getMapping(const VertexBuffer* );

private:
	void				processMesh(Mesh* );

	unsigned int		addVertexBuffers(const VbTuple& );
	unsigned int		addIndexBuffer(IndexBuffer* ib, unsigned int shift);

	void				mergeVertexBuffers();
	void				mergeIndexBuffers();

	void				normalizeFormat(VertexFormat* );
};

}; };

#endif
