/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MODEL_H
#define __GRAPHICS3D_MODEL_H

#include "graphics3d.resource.h"
#include "graphics3d.parameterformat.h"
#include "geometry.bounds.h"

namespace magnum { namespace graphics3d {
using magnum::geometry::Bounds3;

class ParameterBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Joint {
private:
	char				m_Name[64];
	unsigned int		m_Index;

public:
	inline Joint();
	inline Joint(const char* name, unsigned int idx);

	inline const char*	getName() const;
	inline unsigned int	getIndex() const;
};

typedef Array<Joint>	JointList;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Model : public Resource {
DeclareClass(Model, Resource);
private:
	bool				m_Optimized;
	ParameterFormat		m_Parameters;
	JointList			m_Joints;
	Bounds3<float>		m_Bounds;
						
protected:				
	inline				Model();
						
public:					
	inline void			render(const ParameterBuffer* ) const;
	inline void			transform(const ParameterBuffer* ,Matrix4<float>* ,unsigned int ) const;

	inline bool			optimize();
	inline bool			isOptimized() const;

	inline const ParameterFormat& getParameters() const;
	inline const JointList& getJoints() const;
	inline const Bounds3<float>& getBounds() const;

protected:
	virtual void		onRender(const ParameterBuffer* ) const = 0;
	virtual void		onTransform(const ParameterBuffer* ,Matrix4<float>* ,unsigned int idx) const;
	virtual bool		onOptimize();
						
	inline void			clearParameters();
	inline void			publishParameters(const ParameterFormat& );

	inline void			clearJoints();
	inline void			publishJoints(const JointList& );
	
	inline void			clearBounds();
	inline void			publishBounds(const Bounds3<float>& );
};

}; };

#endif
