/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MODEL_INL
#define __GRAPHICS3D_MODEL_INL


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Model::Model() {
	m_Optimized = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::render(const ParameterBuffer* pbuf) const {
	onRender(pbuf);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::transform(const ParameterBuffer* pbuf,Matrix4<float>* m, unsigned int idx) const {
	onTransform(pbuf, m, idx);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Model::isOptimized() const {
	return m_Optimized;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Model::optimize() {
	if (m_Optimized)
		return false;
	m_Optimized = onOptimize();
	return m_Optimized;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const ParameterFormat& Model::getParameters() const {
	return m_Parameters;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::clearParameters() {
	m_Parameters.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Model::publishParameters(const ParameterFormat& fmt) {
	m_Parameters.append(fmt);
	m_Parameters.shrink();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const JointList& Model::getJoints() const {
	return m_Joints;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::clearJoints() {
	m_Joints.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Model::publishJoints(const JointList& fmt) {
	m_Joints.append(fmt);
	m_Joints.shrink();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Bounds3<float>& Model::getBounds() const {
	return m_Bounds;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::clearBounds() {
	m_Bounds.empty();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::publishBounds(const Bounds3<float>& b) {
	m_Bounds = b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Joint::Joint() {
	m_Name[0] = 0;
	m_Index = ~0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Joint::Joint(const char* name, unsigned int idx) {
	strncpy(m_Name, name, sizeof(m_Name));
	m_Index = idx;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const char* Joint::getName() const {
	return m_Name;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Joint::getIndex() const {
	return m_Index;
}

}; };

#endif
