/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_MODESTACK_INL
#define __GRAPHICS3D_MODESTACK_INL

namespace magnum {namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline ModeStack::ModeStack() {
	m_Mode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Mode* ModeStack::getMode() const {
	return m_Mode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setDepthRange(float min, float max) {
	glDepthRange(min, max);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setDepthRange(const Range<float>& r) {
	glDepthRange(r.getStart(), r.getEnd());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setDepthTest(Mode::TestFunction t) {
	if (t == Mode::Disable) {
		glDisable(GL_DEPTH_TEST);
	} else {
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(t);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setDepthWrite(bool b) {
	glDepthMask(b);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setStencilMode(Mode::StencilMode fail,Mode::StencilMode zfail,Mode::StencilMode pass) {
	glStencilOp(fail, zfail, pass);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setStencilTest(Mode::TestFunction t,unsigned int ref, unsigned int mask) {
	m_StencilTest = t;
	m_StencilRef = ref;
	m_StencilMask = mask;
	if (m_StencilTest == Mode::Disable) {
		glDisable(GL_STENCIL_TEST);
	} else {
		glEnable(GL_STENCIL_TEST);
		glStencilFunc(t, ref, mask);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setStencilReference(unsigned int ref) {
	m_StencilRef = ref;
	glStencilFunc(m_StencilTest, ref, m_StencilMask);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setStencilWrite(unsigned int mask) {
	glStencilMask(mask);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setBlendMode(Mode::BlendMode src, Mode::BlendMode dst) {
	if ((src == Mode::One) && (dst == Mode::Zero)) {
		glDisable(GL_BLEND);
	} else {
		glEnable(GL_BLEND);
		glBlendFunc(src ,dst);
	}
}
						

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ModeStack::setBlendEquation(Mode::Equation e) {
	ASSERT(glBlendEquationEXT != NULL);
	glBlendEquationEXT(e);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setBlendColour(const Colour<float>& col) {
	ASSERT(glBlendColorEXT != NULL);
	glBlendColorEXT(col.R, col.G, col.B, col.A);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setColourWrite(bool r, bool g, bool b, bool a) {
	glColorMask(r, g, b, a);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setColourWrite(bool r) {
	glColorMask(r, r, r, r);
}
		

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ModeStack::setAlphaTest(Mode::TestFunction t, float ref) {
	if (t == Mode::Disable) {
		glDisable(GL_ALPHA_TEST);
	} else {
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(t, ref);
	}
}

}; };

#endif
