#ifndef __GRAPHICS3D_PARAMETER_INL
#define __GRAPHICS3D_PARAMETER_INL

namespace magnum {namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline ParameterBuffer::ParameterBuffer() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	ParameterBuffer::getSize() const {
	return m_Format->getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* ParameterBuffer::getPointer(Location l) {
	ASSERT(m_Format->contains(l));
	return (void*)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const void* ParameterBuffer::getPointer(Location l) const {
	ASSERT(m_Format->contains(l));
	return (void*)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* ParameterBuffer::getBuffer() {
	return m_Parameters;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const void* ParameterBuffer::getBuffer() const {
	return m_Parameters;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const ParameterFormat* ParameterBuffer::getFormat() const {
	return m_Format;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setInteger(Location l,int val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Integer);
	*(int *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int ParameterBuffer::getInteger(Location l) const {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Integer);
	return *(int *)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setFloat(Location l,float val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Float);
	*(float *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float ParameterBuffer::getFloat(Location l) const {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Float);
	return *(float *)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setDouble(Location l,double val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Double);
	*(double *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline double ParameterBuffer::getDouble(Location l) const {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Double);
	return *(double *)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setVector(Location l,const Vector3<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Vector3f);
	*(Vector3<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setVector(Location l,const Vector4<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Vector4f);
	*(Vector4<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setVector(Location l,const Vector3<double>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Vector3d);
	*(Vector3<double> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setVector(Location l,const Vector4<double>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Vector4d);
	*(Vector4<double> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setMatrix(Location l,const Matrix3<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Matrix3f);
	*(Matrix3<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setMatrix(Location l,const Matrix4<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Matrix4f);
	*(Matrix4<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setMatrix(Location l,const Matrix3<double>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Matrix3d);
	*(Matrix3<double> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setMatrix(Location l,const Matrix4<double>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Matrix4d);
	*(Matrix4<double> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setColour(Location l,const Colour<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Colour4f);
	*(Colour<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setColour(Location l,const Colour<byte>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT((*m_Format)[l].getType() == Types::Colour4b);
	*(Colour<byte> *)(void *)((byte *)m_Parameters + l) = val;
}

}; };

#endif
