#ifndef __GRAPHICS3D_PARAMETERFORMAT_H
#define __GRAPHICS3D_PARAMETERFORMAT_H

#include "graphics3d.types.h"
#include "graphics3d.parameter.h"
#include "container.array.h"

namespace magnum { namespace graphics3d {
using magnum::container::Array;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ParameterFormat : public Object {
public:
	class Component {
	friend ParameterFormat;
	private:
		char				m_Name[64];
		char				m_Description[64];
		unsigned int		m_Location;
		Types::Type			m_Type;
						
	public:				
		inline const char*	getName() const;
		inline const char*	getDescription() const;
		inline Types::Type	getType() const;
		inline ParameterBuffer::Location getLocation() const;
	};
	
	typedef magnum::container::Array<Component>::Iterator	Iterator;
	
private:
	Array<Component>		m_Components;
	unsigned int			m_Length;
	
public:
	inline ParameterFormat();

	inline void				clear();
	void					insert(const char* ,const char* ,Types::Type ,ParameterBuffer::Location );
	void					insert(const char* ,Types::Type ,ParameterBuffer::Location );
	void					remove(const char* );
	void					shrink();
	bool					contains(const char* ) const;
	bool					contains(ParameterBuffer::Location ) const;
	void					append(const ParameterFormat& );
							
	inline unsigned int		getSize() const;
	inline unsigned int		getLength() const;
							
	inline const char*		getDescription(const char* ) const;
	inline Types::Type		getType(const char* ) const;
	inline unsigned int		getSize(const char* ) const;
	inline ParameterBuffer::Location getLocation(const char* ) const;
							
	const Component& 		operator[](const char* name) const;
	const Component& 		operator[](ParameterBuffer::Location ) const;
							
	inline Iterator 		begin() const;
	Iterator		 		at(const char* name) const;
	Iterator 				at(ParameterBuffer::Location ) const;
	inline Iterator 		end() const;
	
	inline const ParameterFormat& operator=(const ParameterFormat&);
	inline bool				operator==(const ParameterFormat& ) const;
	inline bool				operator!=(const ParameterFormat& ) const;
};

}; };

#endif
