/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3d_PROJECTIONVOLUME_H
#define __GRAPHICS3d_PROJECTIONVOLUME_H

#include "container.array.h"
#include "graphics3d.vertexformat.h"
#include "math.vector3.h"
#include "math.vector4.h"
#include "geometry.trianglemesh.h"


namespace magnum { namespace graphics3d {
using magnum::math::Vector3;
using magnum::math::Vector4;
using magnum::geometry::TriangleMesh3;
using magnum::container::Array;
using magnum::types::int8;

class IndexBuffer;
class VertexBuffer;
class Mesh;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ProjectionVolume : public Object {
DeclareClass(ProjectionVolume, Object);
private:
	struct Edge {
		unsigned int	m_Normal[2];
		unsigned int	m_Vertex[2];
	};

	typedef Vector3<float> vector3;
	typedef Vector4<float> vector4;

	Array<vector3>		m_Vertices;
	Array<vector4>		m_Normals;
	Array<int8>			m_Signs;
	Array<Edge>			m_Edges;

	VertexFormat		m_VertexFormat;
	unsigned int		m_EdgeCount;
	unsigned int		m_FaceCount;
	unsigned int		m_VertexCount;
	unsigned int		m_IndexRange;
	VertexBuffer*		m_VertexBuffer;
	IndexBuffer*		m_IndexBuffer;
	TriangleMesh3<float> m_TriangleMesh;

public:
	ProjectionVolume();
	~ProjectionVolume();

	void				clear();
	bool				add(const Mesh* );
	bool				add(const TriangleMesh3<float>* );
	void				finish();
						
	void				update(const Vector3<float>& lightpos, float lsize);
						
	inline unsigned int	getIndexRange();
	inline IndexBuffer*	getIndexBuffer();
	inline VertexBuffer* getVertexBuffer();
};

}; };

#endif
