/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_SHADER_INL
#define __GRAPHICS3D_SHADER_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Shader::Shader() {
	m_Attributes = 0;
	m_BaseAttributes = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Shader::select() const {
	ShaderStack* ss = getContext()->getShaderStack();
	const Shader* cur = ss->m_Shader;
	if (cur == this)
		return;
	ss->m_Shader = this;
	if (cur != NULL) {
//		if (typeid(*cur) != typeid(*this))
			cur->onDeselect();
	}
	selectAttributes();
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Shader::forceSelect() const {
	ShaderStack* ss = getContext()->getShaderStack();
	const Shader* cur = ss->m_Shader;
	ss->m_Shader = this;
	if (cur != NULL) {
//		if (typeid(*cur) != typeid(*this))
			cur->onDeselect();
	}
	selectAttributes();
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Shader::deselect() const {
	ShaderStack* ss = getContext()->getShaderStack();
	if (ss->m_Shader == this) {
		ss->m_Shader = NULL;
		onDeselect();
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Shader::selectNone() {
	ShaderStack* ss = getContext()->getShaderStack();
	if (ss->m_Shader != NULL) {
		ss->m_Shader->onDeselect();
		ss->m_Shader = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Shader::attachPixelProgram(PixelProgram* prg) {
	m_PixelProgram = prg;
	updateAttributes();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Shader::detachPixelProgram() {
	m_PixelProgram = NULL;
	updateAttributes();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline PixelProgram* Shader::getPixelProgram() const {
	return m_PixelProgram;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Shader::attachVertexProgram(VertexProgram* prg) {
	m_VertexProgram = prg;
	updateAttributes();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Shader::detachVertexProgram() {
	m_VertexProgram = NULL;
	updateAttributes();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexProgram* Shader::getVertexProgram() const {
	return m_VertexProgram;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::publishAttributes(unsigned int att) {
	m_BaseAttributes = att;
	updateAttributes();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Shader::getAttributes() const {
	return m_Attributes;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const ConstantList& Shader::getConstants() const {
	return m_Constants;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Shader::publishConstants(const ConstantList& sc) {
	m_Constants = sc;
}

}; };

#endif
