/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_3DS_H
#define __GRAPHICS3D_TRANSLATOR_3DS_H

#include "object.pointer.h"
#include "io.translator.h"
#include "container.map.h"
#include "container.stack.h"
#include "container.array.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::graphics::Colour;
using magnum::container::Map;
using magnum::container::Stack;
using magnum::container::Array;

class StaticModel;
class IndexMesh;
class Shader;
class Appearance;
class VertexBuffer;
class IndexBuffer;
class ColourMaterial;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Translator3DS : public ModelTranslator {
DeclareClass(Translator3DS, ModelTranslator);
private:
	Shader*				m_PlainShader;
	Shader*				m_TextureShader;
	InputStream*		m_In;
	StaticModel*		m_Model;
	ColourMaterial*		m_Material;
	Appearance*			m_Appearance;
	VertexBuffer*		m_VertexBuffer;
	IndexBuffer*		m_IndexBuffer;

	Matrix4<float>		m_Matrix;
	float32*			m_VertexList;
	float32*			m_MapList;
	word*				m_FaceList;
	unsigned int		m_VertexCount;
	unsigned int		m_FaceCount;

	struct FaceMaterial {
		Appearance*		m_Appearance;
		int				m_FaceCount;
		word*			m_FaceList;
	};

	Array<FaceMaterial>					m_FaceMaterials;
	Map<String, Pointer<Appearance> >	m_Materials;

#pragma pack(push,1)
	struct Chunk {
		word			m_Id;
		dword			m_Len;
	};					
#pragma pack(pop)		
						
	Stack<Chunk>		m_ChunkStack;
	dword				m_ChunkLen;
	word				m_ChunkId;
	long				m_ChunkLeft;

protected:
	Model*			onDecode(InputStream* ,long size = Autosize);
	long			onEncode(OutputStream* ,const Model* );
	bool			onIdentify(File* );


private:
	void			ascendChunk();
	bool			nextChunk();
	void			descendChunk();

	inline long		getChunkSize();
	inline word		getChunkId();

	inline long		readChunk(void* dst, long len);

	Colour<float>	readColour();
	String			readString();
	float			readScalar();

	void			readMaterial();
	void			readMaterialTexture();
	void			readVertexList();
	void			readFaceList();
	void			readFaceMaterial();
	void			readMapList();
	void			readTraMatrix();
	void			readTriMesh();
	void			readObjectBlock();
	void			readMeshWorld();

	void			createVertexBuffer();
	void			createIndexBuffer();
	void			createMeshes();
};

}; };

#endif

