/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_H
#define __GRAPHICS3D_TRANSLATOR_H

#include "graphics3d.model.h"
#include "io.translator.h"
#include "object.pointer.h"


namespace magnum { namespace graphics3d {
using magnum::object::PropertyList;
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::io::File;
using magnum::io::InputStream;
using magnum::io::OutputStream;

class TextureManager;
class ShaderManager;
class Texture;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ModelTranslator : public magnum::io::Translator<Model> {
private:
	Pointer<TextureManager>		m_TextureManager;
	Pointer<ShaderManager>		m_ShaderManager;

public:
	inline void				attachTextureManager(TextureManager* );
	inline void				detachTextureManager();
	inline TextureManager*	getTextureManager() const;

	inline void				attachShaderManager(ShaderManager* );
	inline void				detachShaderManager();
	inline ShaderManager*	getShaderManager() const;
	
protected:
	Texture*				createTexture(const char* );

	virtual PropertyList*	onCreateProperties() const;
	virtual void			onSetProperties(const PropertyList* );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ModelTranslatorRoster : public magnum::io::TranslatorRoster<Model> {
private:
	Pointer<TextureManager>		m_TextureManager;
	Pointer<ShaderManager>		m_ShaderManager;

public:
	void					attachTextureManager(TextureManager* );
	void					detachTextureManager();
	inline TextureManager*	getTextureManager() const;

	void					attachShaderManager(ShaderManager* );
	void					detachShaderManager();
	inline ShaderManager*	getShaderManager() const;
};


}; };

#endif
