/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GRAPHICS3D_VERTEXFORMAT_INL
#define __GRAPHICS3D_VERTEXFORMAT_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexFormat::VertexFormat() {
	m_Size = 0;
	m_Stride = 0;
	memset(&m_Component, 0, sizeof(m_Component));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexFormat::VertexFormat(const VertexFormat& vf) {
	m_Size = vf.m_Size;
	m_Stride = vf.m_Stride;
	memcpy(&m_Component, &vf.m_Component, sizeof(m_Component));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void VertexFormat::clear() {
	m_Size = 0;
	m_Stride = 0;
	memset(&m_Component, 0, sizeof(m_Component));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexFormat::Type VertexFormat::Component::getType() const {
	return m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexFormat::Attribute VertexFormat::Component::getAttribute() const {
	return m_Attribute;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int VertexFormat::Component::getOffset() const {
	return m_Offset;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int VertexFormat::Component::getSize() const {
	return ((int)m_Type >> Types::SizeShift) & Types::SizeMask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int VertexFormat::Component::getCount() const {
	return ((int)m_Type >> Types::CountShift) & Types::CountMask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const VertexFormat::Component& VertexFormat::at(unsigned int i) const {
	ASSERT(i < MaxComponents);
	return m_Component[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexFormat::Iterator VertexFormat::begin() const {
	return (Iterator)&m_Component[0];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexFormat::Iterator VertexFormat::end() const {
	return (Iterator)&m_Component[m_Size];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int VertexFormat::getStride() const {
	return m_Stride;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int VertexFormat::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int VertexFormat::getOffset(unsigned int i) const {
	ASSERT(i < m_Size);
	return m_Component[i].getOffset();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexFormat::Attribute VertexFormat::getAttribute(unsigned int i) const {
	ASSERT(i < m_Size);
	return m_Component[i].getAttribute();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline VertexFormat::Type VertexFormat::getType(unsigned int i) const {
	ASSERT(i < m_Size);
	return m_Component[i].getType();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int VertexFormat::getSize(unsigned int i) const {
	ASSERT(i < m_Size);
	return (m_Component[i].getType() >> Types::SizeShift ) & Types::SizeMask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int VertexFormat::getCount(unsigned int i) const {
	ASSERT(i < m_Size);
	return (m_Component[i].getType() >> Types::CountShift ) & Types::CountMask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const VertexFormat& VertexFormat::operator=(const VertexFormat& vf) {
	m_Size = vf.m_Size;
	m_Stride = vf.m_Stride;
	memcpy(&m_Component, &vf.m_Component, sizeof(m_Component));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const VertexFormat::Component& VertexFormat::operator[](unsigned int i) const {
	return m_Component[i];
}


}; };

#endif
