/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GUI_OPENGL_H
#define __GUI_OPENGL_H

#include "gui.view.h"
#include "graphics3d.context.h"

#if defined(__FORWINDOWS__)
#	pragma comment (lib, "winmm.lib")     /* link with Windows MultiMedia lib */
#	pragma comment (lib, "opengl32.lib")  /* link with Microsoft OpenGL lib */
#	pragma comment (lib, "glu32.lib")     /* link with OpenGL Utility lib */
#elif defined(__FORX11__)
#	include <X11/Xlib.h>
#	include <GL/gl.h>
#	include <GL/glx.h>
#endif

namespace magnum { namespace gui {
using magnum::object::Object;
using magnum::types::String;
using magnum::graphics::Rectangle;
using magnum::graphics::Dimension;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class OpenGL : public View {
DeclareClass(OpenGL, View);
private:
	magnum::graphics3d::Context* m_Context;
	int				m_ColourBits;
	int				m_AlphaBits;
	int				m_StencilBits;
	int				m_AccumulationBits;
	int				m_DepthBits;
	int				m_AuxBits;


public:
	OpenGL(const Rectangle<float>& rect);
	~OpenGL();

	bool			verifyFormat() const;
	bool			adjustFormat();
					
	inline int		getColourBits() const;
	inline void		setColourBits(int bits);
	inline int		getAlphaBits() const;
	inline void		setAlphaBits(int bits);
	inline int		getStencilBits() const;
	inline void		setStencilBits(int bits);
	inline int		getAccumulationBits() const;
	inline void		setAccumulationBits(int bits);
	inline int		getDepthBits() const;
	inline void		setDepthBits(int bits);
	inline int		getAuxBits() const;
	inline void		setAuxBits(int bits);
	
	inline magnum::graphics3d::Context* getContext() const;
	
protected:
	virtual void	onRender();
					
	virtual bool	onResize(const Dimension<float>& );
	virtual bool	onAttachWindow();
	virtual bool	onDetachWindow();


private:
#if defined(__FORWINDOWS__)
	HGLRC			m_OpenGLContext;
	HPALETTE		m_PaletteHandle;
					
	int				chooseFormat(HDC deviceHandle, PIXELFORMATDESCRIPTOR* pfd) const;
	bool			adjustFormat(HDC deviceHandle, PIXELFORMATDESCRIPTOR* pfd);

	static LRESULT CALLBACK windowProc(HWND hwnd, UINT Message,WPARAM wParam, LPARAM lParam);
#elif defined(__FORX11__)
	GLXContext		m_OpenGLContext;
	Window			m_WindowHandle;
	Display*		m_Display;

	XVisualInfo*	chooseFormat(Display* display) const;
	XVisualInfo*	adjustFormat(Display* display);

protected:
	bool			dispatchEvent(const XEvent& event);
#endif
};

}; };

#endif
