/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GUI3D_CONTEXT_H
#define __GUI3D_CONTEXT_H

#include "object.h"
#include "object.pointer.h"
#include "graphics.transform.h"
#include "graphics.rectangle.h"
#include "graphics.image.h"
#include "graphics3d.context.h"
#include "gui3d.bitmap.h"
#include "gui3d.fontcache.h"

namespace magnum { namespace gui3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::graphics::Colour;
using magnum::graphics::Rectangle;
using magnum::graphics::Point;
using magnum::graphics::Dimension;
using magnum::graphics::Transform;

class Root;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Context : public Object {
DeclareClass(Context, Object);
friend class Root;
private:
	enum DrawMode {
		None			= 0,
		Rectangles		= 1,
		Lines			= 2,
		Points			= 3,
		Polygons		= 4,
		Textures		= 5,
		Strings			= 6,
	};
	Pointer<magnum::graphics3d::Context>	m_Context;
	magnum::graphics3d::MatrixStack*		m_MatrixStack;
	magnum::graphics3d::ModeStack*			m_ModeStack;
	magnum::graphics3d::FrameBuffer*		m_FrameBuffer;
	DrawMode								m_Mode;
	Dimension<int>							m_Size;
	const Bitmap*							m_Bitmap;
	FontCache								m_FontCache;
	const FontMap*							m_Font;
	Colour<float>							m_Colour;
	
private:
	Context(magnum::graphics3d::Context* );	

public:
	void				start();
	void				finish();
	
	void				drawRectangle(const Rectangle<float>& );
	void				fillRectangle(const Rectangle<float>& );
	void				drawPoint(const Point<float>& );
	void				drawPoints(const Point<float>* , unsigned int n);
	void				drawLine(const Point<float>& , const Point<float>& );
	void				drawLines(const Point<float>* , unsigned int n);
	void				drawPolygon(const Point<float>* , unsigned int n);
	void				fillPolygon(const Point<float>* , unsigned int n);

	inline FontMap*		createFont(const Font* );
	inline void			selectFont(const FontMap* );
	inline void			deselectFont();
	inline const FontMap* getFont() const;
	void				drawString(const Point<float>& ,const char* );
	Rectangle<float>	getStringExtents(const char* ) const;

	inline void			selectBitmap(const Bitmap* );
	inline void			deselectBitmap();
	inline const Bitmap* getBitmap() const;
	void				drawBitmap(const Point<float>& );
	void				drawBitmap(const Point<float>& ,const Rectangle<float>& );
	void				drawBitmap(const Rectangle<float>& );
	void				drawBitmap(const Rectangle<float>& ,const Point<float>& );

	inline magnum::graphics3d::Context* getContext() const;

	inline const Colour<float>& getColour() const;
	inline void			setColour(const Colour<float>& );
	
	inline void			setLineWidth(float );

	Transform<float>	getTransform() const;
	void				setTransform(const Transform<float>& );

	inline void			setClipping(const Rectangle<int>& );
	Rectangle<int>		getClipping() const;
	
private:
	void				resetMode();
};

}; };

#endif
