/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GUI3D_CONTEXT_INL
#define __GUI3D_CONTEXT_INL

namespace magnum { namespace gui3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline magnum::graphics3d::Context* Context::getContext() const {
	return m_Context;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::setClipping(const Rectangle<int>& rect) {
	resetMode();
	m_FrameBuffer->setClipRectangle(rect);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::setTransform(const Transform<float>& t) {
	resetMode();
	float mtx[4][4];
	mtx[0][0] = t[0][0];
	mtx[1][0] = t[0][1];
	mtx[3][0] = t[0][2];
	mtx[0][1] = t[1][0];
	mtx[1][1] = t[1][1];
	mtx[3][1] = t[1][2];
	mtx[2][2] = 
	mtx[3][3] = 1.0f;
	mtx[2][0] = 
	mtx[2][1] = 
	mtx[0][2] = 
	mtx[1][2] = 
	mtx[3][2] = 
	mtx[0][3] = 
	mtx[1][3] = 
	mtx[2][3] = 0.0f;
	glLoadMatrixf(&mtx[0][0]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::selectBitmap(const Bitmap* bmp) {
	ASSERT(bmp != NULL);
	if (m_Mode != None)
		resetMode();
	m_Bitmap = bmp;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::deselectBitmap() {
	if (m_Mode != None)
		resetMode();
	m_Bitmap = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Bitmap* Context::getBitmap() const {
	return m_Bitmap;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FontMap* Context::createFont(const Font* font) {
	return m_FontCache.get(font);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::selectFont(const FontMap* font) {
	ASSERT(font != NULL);
	if (m_Mode != None)
		resetMode();
	m_Font = font;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::deselectFont() {
	if (m_Mode != None)
		resetMode();
	m_Font = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const FontMap* Context::getFont() const {
	return m_Font;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::setColour(const Colour<float>& c) {
	m_Colour = c;
	glColor3f(c.R, c.G, c.B);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Colour<float>& Context::getColour() const {
	return m_Colour;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::setLineWidth(float w) {
	if (m_Mode != None)
		resetMode();
	glLineWidth(w);
}
}; };

#endif
