/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __GUI3D_VIEW_H
#define __GUI3D_VIEW_H

#include "container.chain.h"
#include "graphics.rectangle.h"
#include "graphics.dimension.h"
#include "graphics.point.h"
#include "graphics.transform.h"
#include "application.keyboard.h"
#include "application.handler.h"

namespace magnum { namespace gui3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Chainable;
using magnum::graphics::Rectangle;
using magnum::graphics::Point;
using magnum::graphics::Dimension;
using magnum::graphics::Transform;
using magnum::application::Mouse;
using magnum::application::Keyboard;
using magnum::application::Character;
using magnum::application::Handler;

class Context;
class Container;
class Root;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class View : public Handler, private Chainable<View> {
DeclareClass(View, Handler);
friend class Container;
friend class Root;
private:
	Root*					m_Root;
	Container*				m_Parent;
	Context*				m_Context;
	String					m_Name;
	Rectangle<float>		m_Bounds;
	Rectangle<float>		m_Frame;
	Transform<float>		m_Transform;
	Transform<float>		m_AbsTransform;
	int						m_Alignement;
	bool					m_Enabled;
	
public:
	enum Alignement {
		FollowLeft = 0x01,
		FollowRight = 0x02,
		FollowLeftRight = 0x03,
		FollowHCenter = 0x04,
		FollowTop = 0x10,
		FollowBottom = 0x20,
		FollowTopBottom = 0x30,
		FollowVCenter = 0x40,
		FollowAll = 0x33
	};
	
public:
	View(const Rectangle<float>& );
	~View();
	
    inline Root*			getRoot() const;
    inline Container*   	getParent() const;
    inline Context*			getContext() const;

    inline void         	setName(const char* );
    inline String       	getName(void) const;

    inline const Rectangle<float>& getFrame() const;
    inline void				setFrame(const Rectangle<float>& );

    inline Dimension<float> getSize() const;
    inline void				setSize(const Dimension<float>& );

    inline Point<float> 	getLocation() const;
    inline void         	setLocation(const Point<float>& );

    inline const Rectangle<float>& getBounds() const;
    void					setBounds(const Rectangle<float>& );

    inline Point<float>		getOrigin() const;
    void					setOrigin(const Point<float>& );

    inline Point<float>		getScale() const;
    void					setScale(const Point<float>& );

    inline bool   			getFocus();
    inline bool				hasFocus() const;
	inline bool				releaseFocus();
	
	void					enable();
	void					disable();
	inline bool				isEnabled() const;
	
	inline bool				isHovered() const;
							
	inline int				getAlignement() const;
    inline void				setAlignement(int );

    virtual Dimension<float> getPreferredSize() const;
    virtual bool			setPreferredSize(void);
    
	inline  Point<float>	convertToParent(const Point<float>& localPoint) const;
	inline  Dimension<float> convertToParent(const Dimension<float>& localDimension) const;
	inline  Rectangle<float> convertToParent(const Rectangle<float>& localRect) const; 

	inline  Point<float>	convertFromParent(const Point<float>& localPoint) const; 
	inline  Dimension<float> convertFromParent(const Dimension<float>& localDimension) const; 
	inline  Rectangle<float> convertFromParent(const Rectangle<float>& localRect) const; 

	inline  Point<float>	convertToRoot(const Point<float>& localPoint) const;
	inline  Dimension<float> convertToRoot(const Dimension<float>& localDimension) const;
	inline  Rectangle<float> convertToRoot(const Rectangle<float>& localRect) const; 

	inline  Point<float>	convertFromRoot(const Point<float>& localPoint) const; 
	inline  Dimension<float> convertFromRoot(const Dimension<float>& localDimension) const; 
	inline  Rectangle<float> convertFromRoot(const Rectangle<float>& localRect) const; 
    
						
protected:				
	// Window management related methods
	virtual bool			onActivate(void);
	virtual bool			onDeactivate(void);
	virtual bool			onAttach();
	virtual bool			onDetach();
	
	// Enable/Disable
	virtual bool			onEnable();
	virtual bool			onDisable();

	// Paint method
	virtual bool			onPaint();
	
	// Size related methods
	virtual bool			onResize(const Dimension<float>& );
    virtual Dimension<float> onGetPreferredSize() const;
    virtual bool			onSetPreferredSize(void);
	
	// Keyboard input related methods
	virtual bool			onGetFocus();
	virtual bool			onReleaseFocus();
	virtual bool			onKeyDown(Keyboard::Key );
	virtual bool			onKeyUp(Keyboard::Key );
	virtual bool			onChar(Character::Code ,Character::Modifier );

	// Mouse related methods
	virtual bool			onMouseMove(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseButton(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseEnter(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseLeave();
	
private:					
	void					updateFrame();
	virtual void			updateTransform();
};

}; };

#endif
