/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __IO_BUFFER_INL
#define __IO_BUFFER_INL


namespace magnum { namespace io {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline InputBuffer::InputBuffer(int bufsize) {
	ASSERT(bufsize > 0);
	m_BufferSize = bufsize;
	m_Buffer = new byte[m_BufferSize];
	m_BufferLeft = 0;
	m_BufferCurrent = m_Buffer;
	m_Stream = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline InputBuffer::InputBuffer(InputStream* in, int bufsize) {
	ASSERT(bufsize > 0);
	ASSERT(in != NULL);
	m_BufferSize = bufsize;
	m_Buffer = new byte[m_BufferSize];
	m_BufferLeft = 0;
	m_BufferCurrent = m_Buffer;
	m_Stream = in;
	m_Stream->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline InputBuffer::InputBuffer(InputStream& in, int bufsize) {
	ASSERT(bufsize > 0);
	m_BufferSize = bufsize;
	m_Buffer = new byte[m_BufferSize];
	m_BufferLeft = 0;
	m_BufferCurrent = m_Buffer;
	m_Stream = &in;
	m_Stream->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline InputBuffer::~InputBuffer() {
	if (m_Buffer)
		delete[] m_Buffer;
	if (m_Stream)
		m_Stream->release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline InputStream*	InputBuffer::getStream() const {
	return m_Stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void InputBuffer::setStream(InputStream* in) {
	ASSERT(in != NULL);
	in->refer();
	if (m_Stream)
		m_Stream->release();
	m_BufferLeft = 0;
	m_BufferCurrent = m_Buffer;
	m_Stream = in;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void InputBuffer::setStream(InputStream& in) {
	in.refer();
	if (m_Stream)
		m_Stream->release();
	m_BufferLeft = 0;
	m_BufferCurrent = m_Buffer;
	m_Stream = &in;
}

}; };

#endif
