/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __IO_MEMFILE_H
#define __IO_MEMFILE_H

#include "io.file.h"
#include "system.timestamp.h"
#include "container.array.h"


namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::String;
using magnum::system::Timestamp;
using magnum::container::Array;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MemoryFile : public File {
DeclareClass(MemoryFile, File);
private:
	Timestamp		m_Timestamp;
	unsigned long	m_Size;
	unsigned long	m_Position;
	int				m_Blocksize;
	Array<void*>	m_Blocks;


public:
	MemoryFile(unsigned long bs = 16384);
	~MemoryFile();

	unsigned long	write(const void *buf, unsigned long size);
	unsigned long	read(void *buf, unsigned long size);
	bool			flush();

	unsigned long	setPosition(long where,PositionMode mode);
	unsigned long	getPosition() const;
	bool			rewind();

	unsigned long	getSize() const;
	unsigned long	setSize(unsigned long newSize);

	bool			isWriteable() const;
	bool			isReadable() const;

	Timestamp		getCreationTime() const;
	Timestamp		getModificationTime() const;
	Timestamp		getAccessTime() const;

private:
	bool			append(unsigned long addSize);
	bool			truncate(unsigned long delSize);
};

}; };

#endif
