/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __IO_SYSFILE_H
#define __IO_SYSFILE_H

#include "io.file.h"

# if defined (__FORWINDOWS__)
# else
#	include <stdio.h>
# endif

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
This class adds support for real physical files on harddiscs, CDs or 
similar mediums. The implementation itself may be highly dependant on
the operating system used. There are three static methods for opening,
creating and closing files. All filenames are not abstract and highly
platform specific.
----------------------------------------------------------------------------*/
class SystemFile : public File { 
DeclareClass(SystemFile, File);
private:
#if defined (__FORWINDOWS__)
	HANDLE	m_FileHandle;
#else
	FILE*	m_FileHandle;
#endif

	bool	m_Writeable;
	bool	m_Readable;

protected:
	SystemFile();

public:
	~SystemFile();

	unsigned long	write(const void *buf, unsigned long size);
	unsigned long	read(void *buf, unsigned long size);
	bool			flush();

	unsigned long	setPosition(long where,PositionMode mode);
	unsigned long	getPosition() const;
	bool			rewind();

	unsigned long	getSize() const;
	unsigned long	setSize(unsigned long newSize);

	bool			isWriteable() const;
	bool			isReadable() const;

	Timestamp		getCreationTime() const;
	Timestamp		getModificationTime() const;
	Timestamp		getAccessTime() const;

	static SystemFile* open(const char *fname, OpenFlags flags = ReadOnly);
	static SystemFile* create(const char *fname);
	static bool		remove(const char *fname);

private:
	static void		throwException();
};

}; };

#endif

