/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __IO_TRANSLATOR_INL
#define __IO_TRANSLATOR_INL


namespace magnum { namespace io {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Translator<T>::Translator() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Translator<T>::~Translator() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T* Translator<T>::decode(File* f, long size) {
	return onDecode(f->getInputStream(), size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline long Translator<T>::encode(File* f, const T* t) {
	return onEncode(f->getOutputStream(), t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T* Translator<T>::decode(InputStream* f, long size) {
	return onDecode(f, size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline long Translator<T>::encode(OutputStream* f, const T* t) {
	return onEncode(f, t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Translator<T>::identify(File* f) {
	return onIdentify(f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline PropertyList* Translator<T>::createProperties() const {
	return onCreateProperties();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Translator<T>::setProperties(const PropertyList* p) {
	onSetProperties(p);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> PropertyList* Translator<T>::onCreateProperties() const {
	return new PropertyList();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> void Translator<T>::onSetProperties(const PropertyList* p) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void TranslatorRoster<T>::insert(Translator<T>* t) {
	m_Translators.insert(t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void TranslatorRoster<T>::remove(Translator<T>* ) {
	m_Translators.remove(t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void TranslatorRoster<T>::clear() {
	m_Translators.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline typename TranslatorRoster<T>::Iterator TranslatorRoster<T>::begin() {
	return m_Translators.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline typename TranslatorRoster<T>::Iterator TranslatorRoster<T>::end() {
	return m_Translators.end();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Chain<typename TranslatorRoster<T>::TranslatorT>& TranslatorRoster<T>::getTranslators() const {
	return m_Translators;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> Translator<T>* TranslatorRoster<T>::identify(File* file) {
	unsigned long oldpos = file->getPosition();
	bool ret;

	magnum::container::Chain<TranslatorT>::Iterator iter;
	for (iter = m_Translators.begin(); iter != m_Translators.end(); iter++) {
		ret = iter->identify(file);
		file->setPosition(oldpos, File::Absolute);
		if (ret)
			return &(*iter);
	}

	return NULL;
}


}; };

#endif
