/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MATH_POLYNOMIAL_H
#define __MATH_POLYNOMIAL_H

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Polynomial {
private:
	unsigned int			m_Degree;
	T*						m_Coefficients;

public:
	inline Polynomial(unsigned int d = 0);
	inline Polynomial(const Polynomial& );
	inline ~Polynomial();

	inline unsigned int		getDegree() const;
	inline void				setDegree(unsigned int );
	unsigned int			getEffectiveDegree() const;

    // assignment
    inline Polynomial&		operator= (const Polynomial& );

    // evaluation
    inline T				operator() (const T& ) const;

    // arithmetic operations
    inline Polynomial		operator+ (const Polynomial& ) const;
    inline Polynomial		operator- (const Polynomial& ) const;
    inline Polynomial		operator* (const Polynomial& ) const;
    inline Polynomial		operator+ (const T&) const; 
    inline Polynomial		operator- (const T&) const; 
    inline Polynomial		operator* (const T&) const;
    inline Polynomial		operator/ (const T&) const;
    inline Polynomial		operator- () const;

    // arithmetic updates
    inline Polynomial&		operator+= (const Polynomial& );
    inline Polynomial&		operator-= (const Polynomial& );
    inline Polynomial&		operator*= (const Polynomial& );
    inline Polynomial&		operator+= (const T&);
    inline Polynomial&		operator-= (const T&);
    inline Polynomial&		operator*= (const T&);
    inline Polynomial&		operator/= (const T&);

    // derivation
    Polynomial				derive() const;

	inline operator			const T*() const;
	inline operator			T*();

	// Compare operator
	bool					operator==(const Polynomial& ) const;
	bool					operator!=(const Polynomial& ) const;
	bool					operator<=(const Polynomial& ) const;
	bool					operator>=(const Polynomial& ) const;
	bool					operator<(const Polynomial& ) const;
	bool					operator>(const Polynomial& ) const;
};

template<class T> inline Polynomial<T> operator*(T ,const Polynomial<T>& );

}; };

#endif
