/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MATH_QUATERNION_H
#define __MATH_QUATERNION_H

#include "math.vector.h"
#include "math.vector3.h"

namespace magnum {namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Quaternion {
public:
	T		w,x,y,z;

public:
	static Quaternion	Identity;
	static Quaternion	Zero;

public:
	inline Quaternion();
	inline Quaternion(const T&, const T&, const T&, const T& );

    inline Quaternion	operator+(const Quaternion& ) const;
    inline Quaternion	operator-(const Quaternion& ) const;
    inline Quaternion	operator*(const Quaternion& ) const;
    inline Quaternion	operator*(const T& ) const;
    inline Quaternion	operator/(const T& ) const;
    inline Quaternion	operator-() const;

    inline Vector3<T>	operator*(const Vector3<T>& ) const;

    inline const Quaternion& operator= (const Quaternion& );
    inline const Quaternion& operator+=(const Quaternion& );
    inline const Quaternion& operator-=(const Quaternion& );
    inline const Quaternion& operator*=(const Quaternion& );
    inline const Quaternion& operator*=(const T& );
    inline const Quaternion& operator/=(const T& );

	inline bool			operator==(const Quaternion& ) const;
	inline bool			operator!=(const Quaternion& ) const;
	inline bool			operator<(const Quaternion& ) const;
	inline bool			operator>(const Quaternion& ) const;
	inline bool			operator<=(const Quaternion& ) const;
	inline bool			operator>=(const Quaternion& ) const;

	inline Quaternion	invert() const;
	inline Quaternion	conjugate() const;
	inline Matrix3<T>	toMatrix() const;
	inline void			fromMatrix(const Matrix3<T>& );

	inline				operator const T*() const;
	inline				operator T*();

	template<class S> inline Quaternion(const Quaternion<S>& q) {
		w = T(q.w);
		x = T(q.x);
		y = T(q.y);
		z = T(q.z);
	};

	template<class S> inline operator Quaternion<S>() const {
		return Quaternion<S>(S(w), S(x), S(y), S(z));
	};
}; 


}; };

#endif
