/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MATH_VECTOR_INL
#define __MATH_VECTOR_INL

namespace magnum {namespace math {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::Vector() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::Vector(const T* s) {
	for (int i = 0; i < dim; i++)
		vec[i] = s[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> template<class S> 
inline Vector<T,dim>::operator Vector<S,dim>() const {
	Vector<S,dim> v;
	for (unsigned int i = 0; i < dim; i++)
		v.vec[i] = (S)vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> template<class S> 
inline Vector<T,dim>::Vector<T,dim>(const Vector<S,dim>& v) {
	for (unsigned int i = 0; i < dim; i++)
		vec[i] = (T)v.vec[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator+(const Vector<T,dim> &u) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] + u.vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator+(const T& c) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] + c;
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator-(const Vector<T,dim> &u) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] - u.vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator-(const T& c) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] - c;
	return v;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator-() const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = -vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator*(const T& s) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] * s;
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> operator*(const T& s,const Vector<T,dim>& in) {
	Vector<T,dim> v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = s * in.vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::operator*(const Vector<T,dim>& v) const {
	float r = vec[0] * v.vec[0];
	for (int i = 1; i < dim; i++)
		r += vec[i] * v.vec[i];
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator/(const T& s) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] / s;
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator+= (const T& s) {
	for (int i = 0; i < dim; i++)
		vec[i] += s;
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator+= (const Vector<T,dim>& v) {
	for (int i = 0; i < dim; i++)
		vec[i] += v.vec[i];
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator-= (const T& s) {
	for (int i = 0; i < dim; i++)
		vec[i] -= s;
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator-= (const Vector<T,dim>& v) {
	for (int i = 0; i < dim; i++)
		vec[i] -= v.vec[i];
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator*= (const T& s) {
	for (int i = 0; i < dim; i++)
		vec[i] *= s;
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator/= (const T& s) {
	for (int i = 0; i < dim; i++)
		vec[i] /= s;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator==(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim; i++) {
		if (vec[i] != v.vec[i])
			return false;
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator!=(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim; i++) {
		if (vec[i] != v.vec[i])
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator<(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim; i++) {
		if (vec[i] < v.vec[i])
			return true;
		if (vec[i] > v.vec[i])
			return false;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator>(const Vector<T,dim>& v) const{
	for (int i = 0; i < dim; i++) {
		if (vec[i] > v.vec[i])
			return true;
		if (vec[i] < v.vec[i])
			return false;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator<=(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim-1; i++) {
		if (vec[i] < v.vec[i])
			return true;
		if (vec[i] > v.vec[i])
			return false;
	}
	return (vec[i] <= v.vec[i]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool	Vector<T,dim>::operator>=(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim-1; i++) {
		if (vec[i] > v.vec[i])
			return true;
		if (vec[i] < v.vec[i])
			return false;
	}
	return (vec[i] >= v.vec[i]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::operator const T*() const {
	return vec;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::operator T*() {
	return vec;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T& Vector<T,dim>::operator()(unsigned int i) {
	ASSERT(i < dim);
	return vec[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const T&	Vector<T,dim>::operator()(unsigned int i) const {
	ASSERT(i < dim);
	return vec[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::length() const {
	T l = vec[0]*vec[0];
	for(unsigned int i = 1; i < dim; i++)
		l += vec[i]*vec[i];
	return T(sqrt(l));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::length2() const {
	T l = vec[0]*vec[0];
	for(unsigned int i = 1; i < dim; i++)
		l += vec[i]*vec[i];
	return l;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::abs() const {
	Vector<T,dim> v;
	for (unsigned int i = 0; i < dim; i++)
		v.vec[i] = abs(vec[i]);
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::max() const {
	T m = vec[0];
	for (unsigned int i = 1; i < dim; i++) {
		if (m < vec[i])
			m = vec[i];
	}
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::min() const {
	T m = vec[0];
	for (unsigned int i = 1; i < dim; i++) {
		if (m > vec[i])
			m = vec[i];
	}
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::maxabs() const {
	T m = magnum::math::abs(vec[0]);
	T a;
	for (unsigned int i = 1; i < dim; i++) {
		a = magnum::math::abs(vec[i]);
		if (m < a)
			m = a;
	}
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::minabs() const {
	T m = magnum::math::abs(vec[0]);
	T a;
	for (unsigned int i = 1; i < dim; i++) {
		a = magnum::math::abs(vec[i]);
		if (m > a)
			m = a;
	}
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>& Vector<T,dim>::null() {
	for (unsigned int i = 0; i < dim; i++)
		vec[i] = T(0);
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>& Vector<T,dim>::normalize() {
	T l = length();
	if (l > T(0)) {
		l = 1/l;
		for (unsigned int i = 0; i < dim; i++)
			vec[i] *= l;
	}
	
	return *this
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> const Vector<T,dim> Vector<T,dim>::Null = _VectorNull<T,dim>();


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> _VectorNull<T,dim>::_VectorNull() {
	null();
}


}; };

#endif
