/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MEDIA_BUFFER_H
#define __MEDIA_BUFFER_H

#include "object.h"
#include "system.mutex.h"
#include "container.queue.h"


namespace magnum { namespace media {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Queue;
using magnum::container::Queueable;
using magnum::object::Object;
using magnum::system::Mutex;

class Node;
class Pin;
class InputPin;
class OutputPin;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Buffer : public Object, public Queueable<Buffer> {
DeclareClass(Buffer, Object);
friend class Node;
private:
	//Mutex			m_Mutex;
	InputPin*		m_Consumer;
	OutputPin*		m_Producer;
	Node*			m_Owner;
	int				m_OwnerData;

protected:
	void*			m_Buffer;
	int				m_BufferSize;

public:
	inline Buffer(Node* owner, int data);
	inline ~Buffer();

	inline bool		createBuffer(int size);
	inline bool		releaseBuffer();

	inline bool		lock();
	inline bool		unlock();

	inline void*	getBuffer() const;
	inline int		getBufferSize() const;
	inline bool		setBufferSize(int s);

	inline bool		getBuffer(void* ,int offset=0, int size=-1) const; 
	inline bool		setBuffer(const void* ,int offset=0, int size=-1); 

	inline InputPin* getConsumer() const;
	inline OutputPin* getProducer() const;
	inline Node* getOwner() const;
	inline int		getOwnerData() const;
};

}; };

#endif
