/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __MEDIAP_ROCESSOR_H
#define __MEDIAP_ROCESSOR_H

#include "media.filter.h"
#include "system.thread.h"

namespace magnum { namespace media {
using magnum::system::Thread;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ProcessorNode : public Filter {
DeclareClass(ProcessorNode, Filter);
private:
	class ProcessorThread : public Thread {
	DeclareClass(ProcessorThread, Thread);
	private:
		ProcessorNode*	m_Node;

	public:
		ProcessorThread(ProcessorNode* );
		~ProcessorThread();

	protected:
		bool			run();
	};
friend class ProcessorThread;

#if defined(__FORWINDOWS__)
	enum {
		MSG_SYS_MESSAGE = WM_USER+100,
		MSG_SYS_WAIT = WM_USER+101
	};
#endif

private:
	ProcessorThread*	m_Thread;
	InputPin			m_InputPin;
	OutputPin			m_OutputPin;


public:
	ProcessorNode(Device* dev=NULL, const char* name=NULL);
	~ProcessorNode();

	inline Thread*		getThread() const;
	inline bool			isAlive() const;


protected:
	// Start/Stop/Pause - this is system and device specific
	virtual bool		onStart();
	virtual bool		onStop();
	virtual bool		onPause();
	virtual bool		onUnpause();

	// Buffer routines
	virtual bool		onReceiveBuffer(InputPin* pin, Buffer* buffer );
	virtual bool		onSendBuffer(OutputPin* pin, Buffer* buffer );
	virtual bool		onRequestBuffer(OutputPin* pin);

	// These are the functions for overloading
	virtual bool		onProcess(const Buffer* input, Buffer* output);

private:
	// Event queue - this is system and device specific
	bool				loop();
	bool				quit();
};

}; };

#endif
