/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __NET_DATAGRAMSOCKET_INL
#define __NET_DATAGRAMSOCKET_INL

namespace magnum { namespace net {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DatagramSocket::DatagramSocket() : Socket(IPPROTO_UDP) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DatagramSocket::DatagramSocket(unsigned int port) : Socket(IPPROTO_UDP) {
	bind(port);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DatagramSocket::DatagramSocket(const Address& localAdr, unsigned int port) : Socket(IPPROTO_UDP) {
	bind(localAdr, port);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DatagramSocket::DatagramSocket(const String& localAdr, unsigned int port) : Socket(IPPROTO_UDP) {
	bind(localAdr, port);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int DatagramSocket::getMaximumMessageSize() {
#if defined(__FORWINDOWS__)
	unsigned int size;
	int optlen = sizeof(size);
	getsockopt(m_Socket, SOL_SOCKET, SO_MAX_MSG_SIZE, (char *)&size, (int *)&optlen);
	return size;
#else
	return (unsigned int)-1;
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DatagramPacket::DatagramPacket() {
#if defined(__FORWINDOWS__)
	m_Buffer.buf = NULL;
	m_Buffer.len = 0;
#else
	m_Buffer = NULL;
	m_Length = 0;
#endif
	m_BufferSize = 0;
	m_Port = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DatagramPacket::~DatagramPacket() {
#if defined(__FORWINDOWS__)
	if (m_Buffer.buf != NULL)
		delete[] m_Buffer.buf;
#else
	if (m_Buffer != NULL)
		delete[] (unsigned char*)m_Buffer;
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Address& DatagramPacket::getAddress() const {
	return m_Address;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DatagramPacket::setAddress(const Address& adr) {
	m_Address = adr;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DatagramPacket::setAddress(const String& adr) {
	m_Address = Address(adr);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DatagramPacket::clearAddress() {
	m_Address.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int DatagramPacket::getPort() const {
	return ntohs((unsigned short)m_Port);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DatagramPacket::setPort(unsigned int port) {
	m_Port = htons((unsigned short)port);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int DatagramPacket::getSize() const {
#if defined(__FORWINDOWS__)
	return m_Buffer.len;
#else
	return m_Length;
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	 DatagramPacket::setSize(unsigned int len) {
#if defined(__FORWINDOWS__)
	if (len > m_BufferSize) {
		if (m_Buffer.buf)
			delete[] (unsigned char *)m_Buffer.buf;
		m_BufferSize = len;
		m_Buffer.buf = (char *)new unsigned char[m_BufferSize+1];
	}

	m_Buffer.len = len;
#else
	if (len > m_BufferSize) {
		if (m_Buffer)
			delete[] (unsigned char *)m_Buffer;
		m_BufferSize = len;
		m_Buffer = (char *)new unsigned char[m_BufferSize+1];
	}

	m_Length = len;
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* DatagramPacket::getData() {
#if defined(__FORWINDOWS__)
	return m_Buffer.buf;
#else
	return m_Buffer;
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	DatagramPacket::setData(const void* src, unsigned int len) {
	setSize(len);
#if defined(__FORWINDOWS__)
	memcpy(m_Buffer.buf, src, len);
#else
	memcpy(m_Buffer, src, len);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DatagramPacket::setData(const String& str) {
	setData(str.getString(), str.getLength()+1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String DatagramPacket::toString() const {
#if defined(__FORWINDOWS__)
	if (m_Buffer.buf == NULL)
		return String();
	((unsigned char *)m_Buffer.buf)[m_Buffer.len] = 0;
	return String((const char *)m_Buffer.buf);
#else
	if (m_Buffer == NULL)
		return String();
	((unsigned char *)m_Buffer)[m_Length] = 0;
	return String((const char *)m_Buffer);
#endif
}

}; };

#endif
