/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __OBJECT_POINTER_INL
#define __OBJECT_POINTER_INL

namespace magnum { namespace object {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Pointer<T>::Pointer() {
	m_Pointer = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Pointer<T>::Pointer(T* ptr) {
	m_Pointer = ptr;
	if (m_Pointer != NULL)
		m_Pointer->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Pointer<T>::Pointer(const Pointer& ptr) {
	m_Pointer = ptr.m_Pointer;
	if (m_Pointer != NULL)
		m_Pointer->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Pointer<T>::~Pointer() {
	if (m_Pointer != NULL)
		m_Pointer->release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Pointer<T>& Pointer<T>::operator=(const Pointer<T>& ptr) {
	if (ptr.m_Pointer != NULL)
		ptr.m_Pointer->refer();
	if (m_Pointer != NULL)
		m_Pointer->release();
	m_Pointer = ptr.m_Pointer;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Pointer<T>& Pointer<T>::operator=(T* ptr) {
	if (ptr != NULL)
		ptr->refer();
	if (m_Pointer != NULL)
		m_Pointer->release();
	m_Pointer = ptr;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T* Pointer<T>::operator->() {
	return m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const T* Pointer<T>::operator->() const {
	return m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Pointer<T>::operator T*() const {
	return m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::isNull() const {
	return (m_Pointer == NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Pointer<T>::setNull() {
	if (m_Pointer != NULL) {
		m_Pointer->release();
		m_Pointer = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator==(const Pointer<T>& lhs) const {
	return (lhs.m_Pointer == m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator!=(const Pointer<T>& lhs) const {
	return (lhs.m_Pointer != m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator<(const Pointer<T>& lhs) const {
	return (lhs.m_Pointer > m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator>(const Pointer<T>& lhs) const {
	return (lhs.m_Pointer < m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator<=(const Pointer<T>& lhs) const {
	return (lhs.m_Pointer >= m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator>=(const Pointer<T>& lhs) const {
	return (lhs.m_Pointer <= m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator==(const T* lhs) const {
	return (m_Pointer == lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator!=(const T* lhs) const {
	return (m_Pointer != lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator==(T* lhs) const {
	return (m_Pointer == lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator!=(T* lhs) const {
	return (m_Pointer != lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator==(intptr_t lhs) const {
	return (m_Pointer == (T*)(void*)lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Pointer<T>::operator!=(intptr_t lhs) const {
	return (m_Pointer != (T*)(void*)lhs);
}

}; };
#endif
