/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __OBJECT_PROPERTY_H
#define __OBJECT_PROPERTY_H

#include "container.array.h"

namespace magnum { namespace object {
using magnum::container::Array;
using magnum::types::String;

class PropertyList;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Property : public Object {
DeclareClass(Property, Object);
friend class PropertyList;
public:
	enum Type {
		Unknown = 0,
		String = 1,
		Integer = 2,
		Double = 3,
		Float = 4,
		Pointer = 5,
		End = 0x7fffffff
	};

	union Data {
		int			m_Integer;
		float		m_Float;
		double		m_Double;
		void*		m_Pointer;
		char*		m_String;
	};

private:
	magnum::types::String m_Name;
	magnum::types::String m_Description;
	Type			m_Type;
	Data			m_Value;
	Data			m_Default;
					
public:
	inline Property();
		   ~Property();

	inline Type			getType() const;
	inline magnum::types::String getName() const;
	inline magnum::types::String getDescription() const;

	inline float		getFloat() const;
	inline double		getDouble() const;
	inline int			getInteger() const;
	inline void*		getPointer() const;
	inline const char*	getString() const;

	inline float		getDefaultFloat() const;
	inline double		getDefaultDouble() const;
	inline int			getDefaultInteger() const;
	inline void*		getDefaultPointer() const;
	inline const char*	getDefaultString() const;

	inline void			setFloat(float val);
	inline void			setDouble(double val);
	inline void			setInteger(int val);
	inline void			setPointer(void* val);
		   void			setString(const char* val);
		   void			setDefault();
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
struct PropertyDescriptor {
	Property::Type		m_Type;
	const char*			m_Name;
	const char*			m_Description;
	Property::Data		m_Default;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class PropertyList : public Object {
DeclareClass(PropertyList, Object);
private:
	Array<Property>		m_Propertys;

public:
	typedef magnum::container::Array<Property>::Iterator	Iterator;

public:
	PropertyList();
	PropertyList(const PropertyDescriptor* );

	Property&			operator[](const char* );
	const Property&		operator[](const char* ) const;

	inline Iterator		begin() const;
		   Iterator		at(const char* name) const;
	inline Iterator		end() const;

	void				append(const PropertyList* );

	bool				contains(const char* name) const;

	void				setDefault();
};

}; };

#endif
