/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __PHYSICS_DOMAIN_H
#define __PHYSICS_DOMAIN_H

#include "ode.h"

#include "object.h"
#include "math.vector3.h"
#include "container.chain.h"

namespace magnum { namespace physics {
using magnum::object::Object;
using magnum::container::Chain;
using magnum::math::Vector3;

class Body;
class Joint;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Domain : public Object {
DeclareClass(Domain, Object);
friend class Body;
friend class Joint;
public:
	enum Mode {
		Accurate = 1,
		Quick = 2,
	};

	typedef magnum::container::Chain<Body>::Iterator	Iterator;

private:
	dWorldID				m_World;
	Mode					m_Mode;
	Chain<Body>				m_Bodies;
	Chain<Joint>			m_Joints;

public:
	inline Domain();
	inline ~Domain();

	inline void				setGravity(float ,float ,float );
	inline void				setGravity(const Vector3<float>& );
	inline Vector3<float>	getGravity() const;

	inline void				setMode(Mode );
	inline Mode				getMode() const;

	inline void				process(float );

	inline bool				isEmpty() const;
	inline Iterator			begin() const;
	inline Iterator			at(const Body* ) const;
	inline Iterator			end() const;
};


}; };

#endif
