/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __PHYSICS_DOMAIN_INL
#define __PHYSICS_DOMAIN_INL

namespace magnum { namespace physics {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Domain::Domain() {
	m_World = dWorldCreate();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Domain::~Domain() {
	dWorldDestroy(m_World);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Domain::setGravity(float x,float y,float z) {
	dWorldSetGravity(m_World, x, y, z);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Domain::setGravity(const Vector3<float>& v) {
	dWorldSetGravity(m_World, v.x, v.y, v.z);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> Domain::getGravity() const {
	Vector3<float> vec;
	dWorldGetGravity(m_World, (float *)vec);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Domain::setMode(Mode m) {
	m_Mode = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Domain::Mode Domain::getMode() const {
	return m_Mode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Domain::process(float f) {
	if (m_Mode == Accurate)
		dWorldStep(m_World, f);
	else
		dWorldQuickStep(m_World, f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Domain::isEmpty() const{
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Domain::Iterator Domain::begin() const {
	return Iterator();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Domain::Iterator Domain::at(const Body* ) const {
	return Iterator();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Domain::Iterator Domain::end() const {
	return Iterator();
}

}; };

#endif
