/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SCENE3D_GROUP_H
#define __SCENE3D_GROUP_H

#include "container.iterator.h"
#include "scene3d.node.h"

namespace magnum {namespace scene3d {
using magnum::container::IteratorBase;

class Enumerator;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Group : public Node {
DeclareClass(Group, Node);
friend class Enumerator;
friend class Node;
private:
	Transform3<float>	m_WorldTransform;
	Node*				m_First;
	Node*				m_Last;
	unsigned int		m_ChildTypes;

public:
	class Iterator : public IteratorBase<Iterator> {
	friend class Group;
	public:
		inline Iterator();

		inline				operator Node*() const;
		inline Node*		operator->() const;
		inline Node&		operator*() const;

		inline Iterator&	operator++();
		inline Iterator		operator++(int );
		inline Iterator&	operator--();
		inline Iterator		operator--(int );

		inline bool			operator==(const Iterator& ) const;
		inline bool			operator!=(const Iterator& ) const;

	protected:
		inline Iterator(Node* cur, Node* prev);

	private:
		Node*			m_Current;
		Node*			m_Previous;
	};

	friend Iterator;

protected:
	inline Group(Node::Type t);

public:
	inline Group();
	inline ~Group();

	void				insert(Node* );
	void				remove(Node* );
	void				remove(const Iterator& );
	void				clear();

	inline bool			isEmpty() const;

	inline Iterator		begin() const;
	Iterator			at(const char* ) const;
	inline Iterator		end() const;

	inline void			setWorldTransform(const Transform3<float>& );
	inline const Transform3<float>& getWorldTransform() const;

protected:
	virtual void		onAttachLocale(Locale* );
	virtual void		onDetachLocale(Locale* );
	virtual void		onTransform(const Transform3<float>& );
	virtual void		onEnable();
	virtual void		onDisable();
	virtual void		onInsert(Node* );
	virtual void		onRemove(Node* );

private:
	static inline Node*	getNext(const Node* );
	static inline Node*	getPrev(const Node* );
	static inline int	getAllTypes(const Node* );
};

}; };

#endif
