/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SCENE3D_NODE_H
#define __SCENE3D_NODE_H

#include "object.h"
#include "geometry.transform.h"

namespace magnum { namespace scene3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::geometry::Transform3;

class Group;
class Root;
class Universe;
class Locale;
class Enumerator;
class Behaviour;
 
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Node : public Object {
DeclareClass(Node, Object);
friend class Group;
friend class Enumerator;
friend class Observer;
friend class Universe;
public:
	enum Type {
		Unknown			= 0x00000000,
		TransformType	= 0x00000001,
		LinkType		= 0x00000002,
		GroupType		= 0x00000004,
		LeafType		= 0x00000008,
		ColliderType	= 0x00000010,
		OccluderType	= 0x00000020,
		GraphicsType	= 0x00000100,
		AudioType		= 0x00010000,
		BehaviourType	= 0x01000000,
		AllTypes		= 0xffffffff,
	};

	enum Flag {
		Enabled			= 0x00000001,
		AutoBounds		= 0x00000002,
		DirtyTransform	= 0x00000004,
		ParentTransform = 0x00000008,
		ActiveBehaviour	= 0x00000100,
		ActiveModifier	= 0x00000200,
		ActiveGeometry	= 0x00000400,
		ActiveSound		= 0x00000800,
	};

private:
	String				m_Name;
	Locale*				m_Locale;
	Group*				m_Parent;
	Node*				m_NextDirty;
	Node*				m_PrevDirty;
	Node*				m_Next;
	Node*				m_Prev;
	Type				m_Type;
	unsigned int		m_Flags;


protected:
	explicit Node(Type t);
	
public:
	~Node();

	inline Type			getType() const;

	inline String		getName() const;
	inline void			setName(const char* );

	inline Group*		getParent() const;

	inline Universe*	getUniverse() const;
	inline Locale*		getLocale() const;
	Node*				getRoot() const;

	inline void			applyParentTransform();
	inline bool			hasParentTransform() const;

	void				enable();
	void				disable();
	inline bool			isEnabled() const;

	inline unsigned int	getFlags() const;
	inline void			setFlag(Flag );
	inline void			clearFlag(Flag );
	inline bool			testFlag(Flag ) const;
	
protected:
	virtual void		onAttachLocale(Locale* );
	virtual void		onDetachLocale(Locale* );
	virtual void		onEnable();
	virtual void		onDisable();
	virtual void		onTransform(const Transform3<float>& );

private:
	void				enqueueUpdateTransform();
	void				dequeueUpdateTransform();
};

}; };

#endif
