/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SCENE3D_OBSERVER_H
#define __SCENE3D_OBSERVER_H

#include "object.pointer.h"
#include "geometry.frustum.h"
#include "scene3d.traverser.h"
#include "scene3d.scheduler.h"
#include "scene3d.audioqueue.h"
#include "scene3d.renderqueue.h"

namespace magnum { namespace scene3d {
using magnum::object::Pointer;
using magnum::container::Scheduler;
using magnum::container::Chainable;
using magnum::container::Chain;
using magnum::geometry::Frustum3;

class CameraNode;
class ListenerNode;
class Audio;
class ActivatorNode;
class Behaviour;
class Graphics;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Observer : public Object {
DeclareClass(Observer, Object);
private:
	/*------------------------------------------------------------------------
	------------------------------------------------------------------------*/
	class GraphicsTraverser : public Traverser<GraphicsTraverser, Graphics> {
	friend class Traverser<GraphicsTraverser, Graphics>;
	private:
		Observer*				m_Observer;
		RenderQueue*			m_RenderQueue;
		const CameraNode*		m_Camera;
		Frustum3<float>			m_Frustum;

	public:
		GraphicsTraverser(Observer* );

		void					traverse();

	private:
		inline void				onVisit(Graphics* ) const;
	};
	friend class GraphicsTraverser;
	
	
	/*------------------------------------------------------------------------
	------------------------------------------------------------------------*/
	class AudioTraverser : public Traverser<AudioTraverser, Audio> {
	friend class Traverser<AudioTraverser, Audio>;
	private:
		Observer*				m_Observer;
		AudioQueue*				m_AudioQueue;
		const ListenerNode*		m_Listener;
		Vector3<float>			m_Position;

	public:
		AudioTraverser(Observer* );

		void					traverse();

	private:
		inline void				onVisit(Audio* ) const;
	};
	friend AudioTraverser;


	/*------------------------------------------------------------------------
	------------------------------------------------------------------------*/
	class BehaviourTraverser : public Traverser<BehaviourTraverser, Behaviour> {
	friend class Traverser<BehaviourTraverser, Behaviour>;
	private:
		Observer*				m_Observer;
		BehaviourScheduler*		m_Scheduler;
		const ActivatorNode*	m_Activator;
		Frustum3<float>			m_Frustum;

	public:
		BehaviourTraverser(Observer* );

		void					traverse();

	private:
		inline void				onVisit(Behaviour* ) const;
	};
	friend BehaviourTraverser;


private:
	Pointer<Universe>			m_Universe;

	AudioQueue					m_AudioQueue;
	RenderQueue					m_RenderQueue;
	BehaviourScheduler			m_Scheduler;
	
	GraphicsTraverser			m_GraphicsTraverser;
	AudioTraverser				m_AudioTraverser;
	BehaviourTraverser			m_BehaviourTraverser;
	Pointer<ActivatorNode>		m_Activator;
	Pointer<CameraNode>			m_Camera;
	Pointer<ListenerNode>		m_Listener;

public:
	explicit Observer(Universe* );
	~Observer();

	inline CameraNode*		getCamera() const;
	inline void				attachCamera(CameraNode* );
	inline void				detachCamera();

	inline ListenerNode*	getListener() const;
	inline void				attachListener(ListenerNode* );
	inline void				detachListener();

	inline ActivatorNode*	getActivator() const;
	inline void				attachActivator(ActivatorNode* );
	inline void				detachActivator();

	inline Universe*		getUniverse() const;

	void					process();
	void					listen();
	void					view();

private:
	void					clearActiveBehaviours();
	void					clearActiveGeometries();
	void					clearActiveModifiers();
};

}; };

#endif
