/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SCENE3D_RENDERQUEUE_H
#define __SCENE3D_RENDERQUEUE_H

#include "geometry.bounds.h"
#include "container.chain.h"

namespace magnum { namespace scene3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Chain;
using magnum::geometry::Bounds3;

class Observer;
class Universe;
class Modifier;
class Geometry;
class CameraNode;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class RenderQueue : public Object {
DeclareClass(RenderQueue, Object);
private:
	Observer*			m_Observer;
	Universe*			m_Universe;
	Chain<Modifier>		m_ActiveModifiers;
	Chain<Geometry>		m_ActiveGeometries;

public:
	RenderQueue(Observer* );
	~RenderQueue();

	inline void			insert(Modifier* );
	inline void			remove(Modifier* );
	inline bool			contains(const Modifier* ) const;
	
	inline void			insert(Geometry* );
	inline void			remove(Geometry* );
	inline bool			contains(const Geometry* ) const;
	
	void				clearGeometries();
	void				clearModifiers();
	void				clear();

	void				view();
	
private:
	void				selectModifiers(const Bounds3<float>& );
	
	inline void			selectModifier(Modifier* );
	inline void			deselectModifier(Modifier* );
	inline void			deselectAllModifiers();	
};

}; };

#endif
