/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SYSTEM_ASSEMBLER_H
#define __SYSTEM_ASSEMBLER_H

#include "container.array.h"

namespace magnum { namespace system {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Array;

class Program;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Assembler : public Object {
DeclareClass(Assembler, Object);
private:
	enum Opcode {
		opThisCall,
		opStdCall,
		opCCall,
		opReturn,
		opParameter,
		opThis,
		opPush,
		opPop,
		opMove,
		opLoadImm,
		opLoadPtr,
		opLoadInd,
		opLoadIndOff,
		opStorePtr,
		opStoreInd,
		opStoreIndOff,
		opLabel,
	};
	struct Command {
		Opcode		m_Opcode;
		union {
			float	m_Float[3];
			int		m_Integer[3];
			void*	m_Pointer[3];
		};
	};

	Array<Command>		m_Commands;
	Array<String>		m_Labels;

public:
	enum Register {
		R0 = 6,			// ESI
		R1 = 7,			// EDI
		T0 = 2,			// EDX
		T1 = 3,			// EBX
		This = 1,		// ECX
		Result = 0		// EAX
	};

public:
	Assembler();
	~Assembler();

	void				clear();
	void				finish();

	Program*			compile();


	void				asmReturn();
	void				asmPush(Register );
	void				asmPop(Register );
	void				asmLoad(Register, int );
	void				asmLoad(Register, const void* );
	void				asmLoad(Register, Register );
	void				asmLoad(Register, Register ,int );
	void				asmMove(Register, Register );
	void				asmStore(void* ,Register );
	void				asmStore(Register ,Register);
	void				asmStore(Register ,int , Register);
	void				asmLabel(const char* label);

	void				asmCall(void (__stdcall *)());

	template<typename P1>	
	void asmCall(void (__stdcall *adr)(P1), P1 p1);
	template<typename P1, typename P2>	
	void asmCall(void (__stdcall *adr)(P1 ,P2 ), P1 p1, P2 p2);
	template<typename P1, typename P2, typename P3>	
	void asmCall(void (__stdcall *adr)(P1, P2, P3), P1 p1, P2 p2, P3 p3);
	template<typename P1, typename P2, typename P3, typename P4>	
	void asmCall(void (__stdcall *adr)(P1, P2, P3, P4), P1 p1, P2 p2, P3 p3, P4 p4);
	template<typename P1, typename P2, typename P3, typename P4, typename P5>
	void asmCall(void (__stdcall *adr)(P1, P2, P3, P4, P5), P1 p1, P2 p2, P3 p3, P4 p4, P5 p5);

	void asmCall(void (__cdecl *)());
	template<typename P1>	
	void asmCall(void (__cdecl *adr)(P1), P1 p1);
	template<typename P1, typename P2>	
	void asmCall(void (__cdecl *adr)(P1, P2), P1 p1, P2 p2);
	template<typename P1, typename P2, typename P3>	
	void asmCall(void (__cdecl *adr)(P1, P2, P3), P1 p1, P2 p2, P3 p3);
	template<typename P1, typename P2, typename P3, typename P4>	
	void asmCall(void (__cdecl *adr)(P1, P2, P3, P4), P1 p1, P2 p2, P3 p3, P4 p4);
	template<typename P1, typename P2, typename P3, typename P4, typename P5>	
	void asmCall(void (__cdecl *adr)(P1, P2, P3, P4, P5), P1 p1, P2 p2, P3 p3, P4 p4, P5 p5);

	template<class T>	
	void asmCall(void (T::*adr)(), T* obj);
	template<class T, typename P1>	
	void asmCall(void (T::*adr)(P1), T* obj,P1 p1);
	template<class T, typename P1, typename P2>	
	void asmCall(void (T::*adr)(P1, P2), T* obj,P1 p1 ,P2 p2);
	template<class T, typename P1, typename P2, typename P3>	
	void asmCall(void (T::*adr)(P1, P2, P3), T* obj,P1 p1 ,P2 p2, P3 p3);
	template<class T, typename P1, typename P2, typename P3, typename P4>	
	void asmCall(void (T::*adr)(P1, P2, P3, P4), T* obj,P1 p1 ,P2 p2, P3 p3, P4 p4);
	template<class T, typename P1, typename P2, typename P3, typename P4, typename P5>	
	void asmCall(void (T::*adr)(P1, P2, P3, P4, P5), T* obj,P1 p1 ,P2 p2, P3 p3, P4 p4, P5 p5);

	template<class T>	
	void asmCall(void (T::*adr)() const, const T* obj);
	template<class T, typename P1>	
	void asmCall(void (T::*adr)(P1) const, const T* obj,P1 p1);
	template<class T, typename P1, typename P2>	
	void asmCall(void (T::*adr)(P1, P2) const, const T* obj,P1 p1 ,P2 p2);
	template<class T, typename P1, typename P2, typename P3>	
	void asmCall(void (T::*adr)(P1, P2, P3) const, const T* obj,P1 p1 ,P2 p2, P3 p3);
	template<class T, typename P1, typename P2, typename P3, typename P4>	
	void asmCall(void (T::*adr)(P1, P2, P3, P4) const, const T* obj,P1 p1 ,P2 p2, P3 p3, P4 p4);
	template<class T, typename P1, typename P2, typename P3, typename P4, typename P5>	
	void asmCall(void (T::*adr)(P1, P2, P3, P4, P5) const, const T* obj,P1 p1 ,P2 p2, P3 p3, P4 p4, P5 p5);
};

}; };

#endif
