/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "application.application.h"

#include "application.application.inl"
#include "application.messenger.inl"
#include "container.queue.inl"
#include "system.mutex.inl"
#include "system.signal.inl"
#include "system.thread.inl"

namespace magnum { namespace application {

Application* Application::g_Application = NULL;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Application::Application() {
	ASSERT(g_Application == NULL);
	g_Application = this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Application::~Application() {
	ASSERT(g_Application == this);
	g_Application = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Application::quit() {
	return postMessage(this, QuitMessage);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Application::run() {
	onRun();
	attachThread(Thread::getCurrent());
	
	while(true) {
		m_MessageSignal.wait();
		m_MessageSignal.reset();
		
		while(true) {	
			Message* msg;
			m_MessageMutex.lock();
			msg = m_MessageQueue.removeHead();
			m_MessageMutex.unlock();
			
			if (msg == NULL)
				break;
			if ((msg->getTarget() == this) && (msg->getCommand() == QuitMessage))
				goto quit;
			msg->getTarget()->handleMessage(*msg);
			msg->release();
		}
	}

quit:	
	onQuit();
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Application::onQuit() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Application::onRun() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Application::onHandleMessage(const Message& ) {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Application::onSendMessage(Message* msg) {
	m_MessageMutex.lock();
	m_MessageQueue.insertHead(msg);
	m_MessageMutex.unlock();
	m_MessageSignal.raise(true);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Application::onPostMessage(Message* msg) {
	m_MessageMutex.lock();
	m_MessageQueue.insert(msg);
	m_MessageMutex.unlock();
	m_MessageSignal.raise();
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Application::onSynchronize() {
	ASSERT(false);
	return false;
}

}; };
