/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "application.message.h"
#include "appliaction.Looper.h"
#include "application.Handler.h"
#include "system.thread.h"
#include "system.mutex.h"
#include "system.signal.h"

#include "object.inl"
#include "object.lass.inl"
#include "types.string.inl"
#include "application.looper.inl"
#include "application.handler.inl"
#include "application.message.inl"
#include "container.chain.inl"
#include "system.thread.inl"
#include "system.mutex.inl"
#include "system.signal.inl"


# if !defined(__FORX11__)
#	error "This file is only for X11"
# endif

namespace magnum { namespace application {
using namespace magnum::system;

Looper::StartupX Looper::g_StartupX;
Display* Looper::g_MessageConnection = NULL;
Atom Looper::g_SystemMessage;
Atom Looper::g_QuitMessage;
Atom Looper::g_ViewPointer;
Atom Looper::g_LooperPointer;
Atom Looper::g_WindowPointer;
Atom Looper::g_IntegerType;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Looper::StartupX::StartupX() {
	Display *display;

	XInitThreads();

	g_MessageConnection = XOpenDisplay(NULL);
	if (g_MessageConnection == NULL)
		return;

	g_SystemMessage = XInternAtom(g_MessageConnection, "GOMEZ::Message", False);
	g_QuitMessage = XInternAtom(g_MessageConnection, "GOMEZ::Quit", False);
	g_ViewPointer = XInternAtom(g_MessageConnection, "GOMEZ::ViewPointer", False);
	g_LooperPointer = XInternAtom(g_MessageConnection, "GOMEZ::LooperPointer", False);
	g_WindowPointer = XInternAtom(g_MessageConnection, "GOMEZ::WindowPointer", False);
	g_IntegerType =  XInternAtom(g_MessageConnection, "INTEGER", False);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Looper::StartupX::~StartupX() {
	if (g_MessageConnection == NULL)
		return;

	XCloseDisplay(g_MessageConnection);
	g_MessageConnection = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::sysLoop() {
	// Create display-connection
	m_Display = XOpenDisplay(NULL);

	// Create Input-Only Message Window
	Window root = XDefaultRootWindow(m_Display);
	int screen = XDefaultScreen(m_Display);
	m_MessageQueue = XCreateWindow(m_Display, root, 0, 0, 1, 1, 0, 0, InputOnly, CopyFromParent, 0, NULL);
	void* data = this;
	XChangeProperty(m_Display, m_MessageQueue, g_LooperPointer, g_IntegerType, 32, PropModeReplace, (unsigned char*)&data, sizeof(data)/4);
	XSync(m_Display, False);

	// Notify all waiting threads
	m_Signal.raise();

	// STart event loop
	XEvent event;
	GMessage* msg;
	GHandler* hnd;
	while(true) {
		XNextEvent(m_Display, &event);
		if (event.xclient.window == m_MessageQueue) {
			if (event.xclient.type != ClientMessage)
				continue;
			if (event.xclient.message_type == g_QuitMessage)
				break;
			if (event.xclient.message_type == g_SystemMessage) {
				msg = (GMessage*)event.xclient.data.l[0];
				hnd = (GHandler*)event.xclient.data.l[1];
				hnd->handleMessage(*msg);
				msg->release();
			}
		} else {
			dispatchEvent(event);
		}
	}

	// Destroy Message Window
	XDestroyWindow(m_Display, m_MessageQueue);
	XCloseDisplay(m_Display);
	m_Display = NULL;		
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::sysStop() {
	XEvent event;
	event.xclient.type = ClientMessage;
	event.xclient.display = m_Display;
	event.xclient.window = m_MessageQueue;
    event.xclient.message_type = g_QuitMessage;
    event.xclient.format = 32;
	XSendEvent(m_Display, m_MessageQueue, False, NoEventMask, &event);
	XSync(m_Display, False);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::sysPostMessage(GHandler* target, GMessage* msg) {
	XEvent event;
	event.xclient.type = ClientMessage;
	event.xclient.display = m_Display;
	event.xclient.window = m_MessageQueue;
    event.xclient.message_type = g_SystemMessage;
    event.xclient.format = sizeof(msg)*8;
    event.xclient.data.l[0] = (long)msg;
    event.xclient.data.l[1] = (long)target;
	XSendEvent(m_Display, m_MessageQueue, False, NoEventMask, &event);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::sysSendMessage(GHandler* target, GMessage* msg) {
	XEvent event;
	event.xclient.type = ClientMessage;
	event.xclient.display = m_Display;
	event.xclient.window = m_MessageQueue;
	event.xclient.message_type = g_SystemMessage;
	event.xclient.format = sizeof(msg)*8;
	event.xclient.data.l[0] = (long)msg;
    event.xclient.data.l[1] = (long)target;
	XSendEvent(m_Display, m_MessageQueue, False, NoEventMask, &event);
	XSync(m_Display, False);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::dispatchEvent(const XEvent& event) {
	return true;
}

}; };
