/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics.image.h"
#include "graphics.filter.shrink.h"

#include "io.filter.inl"
#include "container.chain.inl"
#include "graphics.filter.inl"
#include "graphics.image.inl"


namespace magnum { namespace graphics {
using magnum::types::byte;
using magnum::types::word;
using magnum::types::dword;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::process(const Image* in) {
	switch (in->getPixelsize()) {
		case 8:
			return resize8((Image *)in);
		case 16:
			return resize16((Image *)in);
		case 24:
			return resize24((Image *)in);
		case 32:
			return resize32((Image *)in);
		default:
			return NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resize8(Image* in) {
	int w = in->getWidth()/2;
	int h = in->getHeight()/2;
	if ((w <= 0) || (h <= 0))
		return NULL;

	Image* out = new Image(in->getType(), w, h);
	if (out == NULL)
		return NULL;

	if (in->isIndexed())
		out->copyPalette(in);

	int out_ls = out->getLinesize();
	int in_ls = in->getLinesize() << 1;
	int in_w = in->getWidth();

	byte* src = (byte *)in->lock();
	byte* dst = (byte *)out->lock();

	int x,y;
	for (y = 0; y < h; y++) {
		for (x = 0; x < w; x++) {
			dst[x] =  src[x << 1];
		}

		dst = (byte *)((byte *)(void *)dst + out_ls);
		src = (byte *)((byte *)(void *)src + in_ls);
	}

	in->unlock();
	out->unlock();
	
	return out;	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resize16(Image* in) {
	int w = in->getWidth()/2;
	int h = in->getHeight()/2;
	if ((w <= 0) || (h <= 0))
		return NULL;

	Image* out = new Image(in->getType(), w, h);
	if (out == NULL)
		return NULL;

	if (in->isIndexed())
		out->copyPalette(in);

	int out_ls = out->getLinesize();
	int in_ls = in->getLinesize() << 1;
	int in_w = in->getWidth();

	word* src = (word *)in->lock();
	word* dst = (word *)out->lock();

	int x,y;
	for (y = 0; y < h; y++) {
		for (x = 0; x < w; x++) {
			dst[x] =  src[x << 1];
		}

		dst = (word*)((byte *)(void *)dst + out_ls);
		src = (word*)((byte *)(void *)src + in_ls);
	}

	in->unlock();
	out->unlock();
	
	return out;	
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resize24(Image* in) {
	int w = in->getWidth()/2;
	int h = in->getHeight()/2;
	if ((w <= 0) || (h <= 0))
		return NULL;

	Image* out = new Image(in->getType(), w, h);
	if (out == NULL)
		return NULL;

	if (in->isIndexed())
		out->copyPalette(in);

	int out_ls = out->getLinesize();
	int in_ls = in->getLinesize() << 1;
	int in_w = in->getWidth();

	byte* src = (byte *)in->lock();
	byte* dst = (byte *)out->lock();

	int x,y;
	for (y = 0; y < h; y++) {
		for (x = 0; x < w; x++) {
			dst[3*x] =  src[6*x];
			dst[3*x + 1] =  src[6*x+1];
			dst[3*x + 2] =  src[6*x+2];
		}

		dst = (byte *)((byte *)(void *)dst + out_ls);
		src = (byte *)((byte *)(void *)src + in_ls);
	}

	in->unlock();
	out->unlock();
	
	return out;	
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resize32(Image* in) {
	int w = in->getWidth()/2;
	int h = in->getHeight()/2;
	if ((w <= 0) || (h <= 0))
		return NULL;

	Image* out = new Image(in->getType(), w, h);
	if (out == NULL)
		return NULL;

	if (in->isIndexed())
		out->copyPalette(in);

	int out_ls = out->getLinesize();
	int in_ls = in->getLinesize() << 1;
	int in_w = in->getWidth();

	dword* src = (dword *)in->lock();
	dword* dst = (dword *)out->lock();

	int x,y;
	for (y = 0; y < h; y++) {
		for (x = 0; x < w; x++) {
			dst[x] =  src[x << 1];
		}

		dst = (dword*)((byte *)(void *)dst + out_ls);
		src = (dword*)((byte *)(void *)src + in_ls);
	}

	in->unlock();
	out->unlock();
	
	return out;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resample8(Image* in) {
	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resample16(Image* in) {
	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resample24(Image* in) {
	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resample32(Image* in) {
	return NULL;
}


}; };
