/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "system.thread.h"
#include "graphics3d.context.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "base.string.inl"
#include "system.thread.inl"
#include "container.iterator.inl"
#include "container.chain.inl"
#include "graphics.rectangle.inl"
#include "graphics3d.context.inl"
#include "graphics3d.texturestack.inl"
#include "graphics3d.materialstack.inl"
#include "graphics3d.lightstack.inl"
#include "graphics3d.matrixstack.inl"
#include "graphics3d.shaderstack.inl"
#include "graphics3d.modestack.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"
#include "graphics3d.framebuffer.inl"


namespace magnum { namespace graphics3d {

Context::Startup Context::g_Startup;
Chain<Context> Context::g_Contexts;

#if defined(__NATIVETLS__)
	__declspec(thread) Context*	Context::g_Current = NULL;
#else
	Thread::Storage	Context::g_Current;
#endif


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context::Startup::Startup() {
#if defined(__NATIVETLS__)

#else
	Context::g_Current = Thread::createTLS();
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context* Context::create() {
	Context* c = new Context();
	if (c == NULL)
		return NULL;

	// Here we should create a device context etc...


	g_Contexts.insert(c);

	return c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Context::makeCurrent() {
#if defined(__FORWINDOWS__)
	BOOL ret = wglMakeCurrent(m_DeviceHandle, m_OglContext);
	if (ret == FALSE)
		return false;
		
	RECT rect;
	GetWindowRect(m_WindowHandle, &rect);
	m_FrameBuffer.m_Size.setWidth(rect.right - rect.left);
	m_FrameBuffer.m_Size.setHeight(rect.bottom - rect.top);
#endif

#if defined(__NATIVETLS__)
	g_Current = this;
#else
	Thread::setTLS(g_Current, this);
#endif

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context* Context::searchCurrent() {
#if defined(__FORWINDOWS__)	
	HGLRC glc = wglGetCurrentContext();
	HDC dc = wglGetCurrentDC();
#elif defined(__FORUNIX__)
#endif

	magnum::container::Chain<Context>::Iterator iter = g_Contexts.begin();
	magnum::container::Chain<Context>::Iterator end = g_Contexts.end();
	while (iter != end) {
		if (glc == iter->m_OglContext)
			break;
		iter++;
	}

	// Didn't we find the thread?
	if (iter == end)
		return createCurrent();

#if defined(__NATIVETLS__)
	g_Current = &(*iter);
#else
	Thread::setTLS(g_Current, &(*iter));
#endif

	return &(*iter);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context* Context::createCurrent() {
	// Create a new Thread object for the current thread...
	Context *c = new Context();

#if defined(__FORWINDOWS__)	
	HGLRC glc = wglGetCurrentContext();
	HDC dc = wglGetCurrentDC();
	c->m_DeviceHandle = dc;
	c->m_OglContext = glc;
	c->m_WindowHandle = WindowFromDC(dc);
#elif defined(__FORUNIX__)
#endif

	g_Contexts.insert(c);

#if defined(__NATIVETLS__)
	g_Current = c;
#else
	Thread::setTLS(g_Current, c);
#endif

	return c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context::Context() {
}

}; };
