/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "math.vector3.h"
#include "graphics3d.context.h"
#include "graphics3d.opengl.h"
#include "graphics3d.geometryrenderer.h"

#include "math.vector3.inl"
#include "geometry.line.inl"
#include "geometry.ray.inl"
#include "geometry.segment.inl"
#include "geometry.sphere.inl"
#include "geometry.cube.inl"
#include "geometry.alignedbox.inl"
#include "geometry.box.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.geometryrenderer.inl"


namespace magnum { namespace graphics3d {
using magnum::math::Vector3;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GeometryRenderer::render(const Line<float, 3>& ) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GeometryRenderer::render(const Segment<float, 3>& ) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GeometryRenderer::render(const Ray<float, 3>& ) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GeometryRenderer::render(const Sphere<float, 3>& ) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GeometryRenderer::render(const Cube<float, 3>& cube) {
	Vector3<float> o = cube.getOrigin();
	float r = cube.getRadius();
	
	glBegin(GL_QUADS);
	// Face 1
	glNormal3f(-1.0f, 0.0f, 0.0f);
	glVertex3f(o.x - r, o.y - r, o.z - r);
	glVertex3f(o.x - r, o.y - r, o.z + r);
	glVertex3f(o.x - r, o.y + r, o.z + r);
	glVertex3f(o.x - r, o.y + r, o.z - r);

	// Face 2
	glNormal3f(+1.0f, 0.0f, 0.0f);
	glVertex3f(o.x + r, o.y - r, o.z - r);
	glVertex3f(o.x + r, o.y + r, o.z - r);
	glVertex3f(o.x + r, o.y + r, o.z + r);
	glVertex3f(o.x + r, o.y - r, o.z + r);

	// Face	3
	glNormal3f(0.0f, -1.0f, 0.0f);
	glVertex3f(o.x - r, o.y - r, o.z - r);
	glVertex3f(o.x + r, o.y - r, o.z - r);
	glVertex3f(o.x + r, o.y - r, o.z + r);
	glVertex3f(o.x - r, o.y - r, o.z + r);

	// Face	4
	glNormal3f(0.0f, 1.0f, 0.0f);
	glVertex3f(o.x - r, o.y + r, o.z - r);
	glVertex3f(o.x - r, o.y + r, o.z + r);
	glVertex3f(o.x + r, o.y + r, o.z + r);
	glVertex3f(o.x + r, o.y + r, o.z - r);

	// Face	5
	glNormal3f(0.0f, 0.0f, -1.0f);
	glVertex3f(o.x - r, o.y - r, o.z - r);
	glVertex3f(o.x - r, o.y + r, o.z - r);
	glVertex3f(o.x + r, o.y + r, o.z - r);
	glVertex3f(o.x + r, o.y - r, o.z - r);

	// Face	6
	glNormal3f(0.0f, 0.0f, 1.0f);
	glVertex3f(o.x - r, o.y - r, o.z + r);
	glVertex3f(o.x + r, o.y - r, o.z + r);
	glVertex3f(o.x + r, o.y + r, o.z + r);
	glVertex3f(o.x - r, o.y + r, o.z + r);
	glEnd();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GeometryRenderer::render(const AlignedBox<float, 3>& box) {
	Vector3<float> mi = box.getMinimum();
	Vector3<float> ma = box.getMaximum();

	glBegin(GL_QUADS);

	// Face 1
	glNormal3f(-1.0f, 0.0f, 0.0f);
	glVertex3f(mi.x, mi.y, mi.z);
	glVertex3f(mi.x, mi.y, ma.z);
	glVertex3f(mi.x, ma.y, ma.z);
	glVertex3f(mi.x, ma.y, mi.z);

	// Face 2
	glNormal3f(1.0f, 0.0f, 0.0f);
	glVertex3f(ma.x, mi.y, mi.z);
	glVertex3f(ma.x, ma.y, mi.z);
	glVertex3f(ma.x, ma.y, ma.z);
	glVertex3f(ma.x, mi.y, ma.z);

	// Face	3
	glNormal3f(0.0f, -1.0f, 0.0f);
	glVertex3f(mi.x, mi.y, mi.z);
	glVertex3f(ma.x, mi.y, mi.z);
	glVertex3f(ma.x, mi.y, ma.z);
	glVertex3f(mi.x, mi.y, ma.z);
	
	// Face	4
	glNormal3f(0.0f, 1.0f, 0.0f);
	glVertex3f(mi.x, ma.y, mi.z);
	glVertex3f(mi.x, ma.y, ma.z);
	glVertex3f(ma.x, ma.y, ma.z);
	glVertex3f(ma.x, ma.y, mi.z);

	// Face	5
	glNormal3f(0.0f, 0.0f, -1.0f);
	glVertex3f(mi.x, mi.y, mi.z);
	glVertex3f(mi.x, ma.y, mi.z);
	glVertex3f(ma.x, ma.y, mi.z);
	glVertex3f(ma.x, mi.y, mi.z);

	// Face	6
	glNormal3f(0.0f, 0.0f, 1.0f);
	glVertex3f(mi.x, mi.y, ma.z);
	glVertex3f(ma.x, mi.y, ma.z);
	glVertex3f(ma.x, ma.y, ma.z);
	glVertex3f(mi.x, ma.y, ma.z);
	glEnd();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GeometryRenderer::render(const Box<float, 3>& box) {
	Vector3<float> o = box.getOrigin();
	Vector3<float> v[4];
	Vector3<float> n;

	glBegin(GL_QUADS);
	
	// Face 1 OK
	v[0] = o - box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[1] = o - box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	v[2] = o + box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	v[3] = o + box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(1)) % ((Vector3<float>)box.getAxis(0));
	n.normalize();
	glNormal3f(n.x, n.y, n.z);
	glVertex3f(v[0].x, v[0].y, v[0].z);
	glVertex3f(v[1].x, v[1].y, v[1].z);
	glVertex3f(v[2].x, v[2].y, v[2].z);
	glVertex3f(v[3].x, v[3].y, v[3].z);

	// Face 2 OK!
	v[0] = o - box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[1] = o - box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	v[2] = o - box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[3] = o - box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(2)) % ((Vector3<float>)box.getAxis(1));
	n.normalize();
	glNormal3f(n.x, n.y, n.z);
	glVertex3f(v[0].x, v[0].y, v[0].z);
	glVertex3f(v[1].x, v[1].y, v[1].z);
	glVertex3f(v[2].x, v[2].y, v[2].z);
	glVertex3f(v[3].x, v[3].y, v[3].z);
	
	// Face 3 OK
	v[0] = o - box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[1] = o + box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[2] = o + box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	v[3] = o - box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(0)) % ((Vector3<float>)box.getAxis(2));
	n.normalize();
	glNormal3f(n.x, n.y, n.z);
	glVertex3f(v[0].x, v[0].y, v[0].z);
	glVertex3f(v[1].x, v[1].y, v[1].z);
	glVertex3f(v[2].x, v[2].y, v[2].z);
	glVertex3f(v[3].x, v[3].y, v[3].z);

	// Face 4 OK
	v[0] = o + box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[1] = o + box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	v[2] = o + box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[3] = o + box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(1)) % ((Vector3<float>)box.getAxis(2));
	n.normalize();
	glNormal3f(n.x, n.y, n.z);
	glVertex3f(v[0].x, v[0].y, v[0].z);
	glVertex3f(v[1].x, v[1].y, v[1].z);
	glVertex3f(v[2].x, v[2].y, v[2].z);
	glVertex3f(v[3].x, v[3].y, v[3].z);

	// Face 5 
	v[0] = o + box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[1] = o + box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	v[2] = o - box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	v[3] = o - box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(2)) % ((Vector3<float>)box.getAxis(0));
	n.normalize();
	glNormal3f(n.x, n.y, n.z);
	glVertex3f(v[0].x, v[0].y, v[0].z);
	glVertex3f(v[1].x, v[1].y, v[1].z);
	glVertex3f(v[2].x, v[2].y, v[2].z);
	glVertex3f(v[3].x, v[3].y, v[3].z);

	// Face 6
	v[0] = o + box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[1] = o - box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[2] = o - box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	v[3] = o + box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(0)) % ((Vector3<float>)box.getAxis(1));
	n.normalize();
	glNormal3f(n.x, n.y, n.z);
	glVertex3f(v[0].x, v[0].y, v[0].z);
	glVertex3f(v[1].x, v[1].y, v[1].z);
	glVertex3f(v[2].x, v[2].y, v[2].z);
	glVertex3f(v[3].x, v[3].y, v[3].z);

	glEnd();
}

}; };

