/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics3d.context.h"
#include "graphics3d.meshstripper.h"
#include "graphics3d.indexmesh.h"
#include "graphics3d.indexbuffer.h"
#include "graphics3d.material.h"
#include "graphics3d.shader.h"
#include "graphics3d.appearance.h"

#include "graphics3d.meshstripper.inl"
#include "graphics3d.mesh.inl"
#include "graphics3d.vertexmesh.inl"
#include "graphics3d.indexmesh.inl"
#include "graphics3d.indexbuffer.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.appearance.inl"
#include "container.array.inl"
#include "container.allocator.inl"
#include "container.iterator.inl"
#include "container.compound.inl"
#include "container.pair.inl"
#include "container.list.inl"
#include "container.priorityheap.inl"
#include "algorithm.sort.inl"
#include "algorithm.find.inl"
#include "algorithm.swap.inl"
#include "math.graph.inl"
#include "object.pointer.inl"


namespace magnum {namespace graphics3d {
using magnum::types::byte;
using magnum::types::word;
using magnum::types::dword;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> static void stripListMesh(T* _t, IndexMesh* mesh) {
	ASSERT(mesh != NULL);
	Array<T>* strip;
	T* list;
	unsigned int size;

	IndexBuffer* oldbuf = mesh->getIndexBuffer();
	ASSERT(oldbuf != NULL);
	list = (T*)oldbuf->lock(IndexBuffer::ReadOnly);
	if (mesh->getRange().getSize() > 0) {
		list += mesh->getRange().getStart();
		size = mesh->getRange().getSize();
	} else {
		size = oldbuf->getSize();
	}

	TriangleStripper<T> stripper;
	stripper.insert(list, size);
	stripper.finish();
	strip = stripper.strip();

	oldbuf->unlock();

	IndexBuffer* newbuf = IndexBuffer::create((IndexBuffer::Type)sizeof(T), strip->getSize(), IndexBuffer::Static);
	newbuf->setBuffer(0, strip->getArray(), strip->getSize());
	mesh->attachIndexBuffer(newbuf);
	mesh->setRange(0, strip->getSize());
	mesh->setPrimitive(Mesh::TriangleStrip);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool MeshStripper::process(IndexMesh* mesh) {
	ASSERT(mesh != NULL);

	if (mesh->getPrimitive() == Mesh::TriangleStrip)
		return true;
	if (mesh->getPrimitive() != Mesh::TriangleList)
		return false;

	IndexBuffer* idxbuf = mesh->getIndexBuffer();
	if (idxbuf == NULL)
		return false;

	switch(idxbuf->getType()) {
		case IndexBuffer::Index8:
			stripListMesh((byte*)0, mesh);
			break;
		case IndexBuffer::Index16:
			stripListMesh((word*)0, mesh);
			break;
		case IndexBuffer::Index32:
			stripListMesh((dword*)0, mesh);
			break;
		default:
			return false;
	}

/*
	newidx = IndexBuffer::create(idxbuf->getType(), ....);
	if (newidx == NULL)
		return false;

	newbuf = newidx->lock(IndexBuffer::Discard);
	memcpy(newbuf, ..., ...);
	newidx->unlock();
*/
	return true;
}


}; };
