/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "system.thread.h"
#include "graphics3d.shader.h"
#include "graphics3d.pixelprogram.h"
#include "graphics3d.vertexprogram.h"
#include "graphics3d.context.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "object.pointer.inl"
#include "system.thread.inl"
#include "system.assembler.inl"
#include "graphics3d.shader.inl"
#include "graphics3d.shaderprogram.inl"
#include "graphics3d.pixelprogram.inl"
#include "graphics3d.vertexprogram.inl"
#include "graphics3d.context.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"
#include "graphics3d.constant.inl"


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::compileSelect(Assembler& as) const {
	Context* c = getContext();
	int offset = (int)((byte*)&c->getShaderStack()->m_Shader - (byte*)c);
	as.asmCall((void (Shader::*)() const)selectAttributes, this);
	as.asmLoad(Assembler::T0, (int)(intptr_t)this);
	as.asmStore(Assembler::R0, offset, Assembler::T0);

	onCompileSelect(as);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::compileDeselect(Assembler& as) const {
	Context* c = getContext();
	int offset = (int)((byte*)&c->getShaderStack()->m_Shader - (byte*)c);
	as.asmLoad(Assembler::T0, NULL);
	as.asmStore(Assembler::R0, offset, Assembler::T0);

	onCompileDeselect(as);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::onCompileSelect(Assembler& as) const {
	as.asmCall((void (Shader::*)() const)Shader::onSelect, this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::onCompileDeselect(Assembler& as) const {
	as.asmCall((void (Shader::*)() const)Shader::onDeselect, this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::onSelect() const {
	ShaderStack* ss = getContext()->getShaderStack();

	if (ss->m_PixelProgram != m_PixelProgram) {
		if (ss->m_PixelProgram != NULL)
			ss->m_PixelProgram->onDeselect();
		ss->m_PixelProgram = m_PixelProgram;
		if (m_PixelProgram != NULL)
			m_PixelProgram->onSelect();
	}

	if (ss->m_VertexProgram != m_VertexProgram) {
		if (ss->m_VertexProgram != NULL)
			ss->m_VertexProgram->onDeselect();
		ss->m_VertexProgram = m_VertexProgram;
		if (m_VertexProgram != NULL)
			m_VertexProgram->onSelect();
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::onDeselect() const {
	ShaderStack* ss = getContext()->getShaderStack();
	if ((ss->m_PixelProgram == m_PixelProgram) && (m_PixelProgram != NULL)) {
		ss->m_PixelProgram = NULL;
		m_PixelProgram->onDeselect();
	}
	if ((ss->m_VertexProgram == m_VertexProgram) && (m_VertexProgram != NULL)) {
		ss->m_VertexProgram = NULL;
		m_VertexProgram->onDeselect();
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::updateAttributes() {
	m_Attributes = m_BaseAttributes;
	if (m_PixelProgram != NULL)
		m_Attributes |= m_PixelProgram->getAttributes();
	if (m_VertexProgram != NULL)
		m_Attributes |= m_VertexProgram->getAttributes();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Shader::selectAttributes() const {
	ShaderStack* ss = getContext()->getShaderStack();
	if (ss->m_Attributes == m_Attributes)
		return;

	unsigned int enable = m_Attributes & ~ss->m_Attributes;
	unsigned int disable = !m_Attributes & ss->m_Attributes;
	ss->m_Attributes = m_Attributes;

	if (disable & Attributes::Vertex)
		glDisableClientState(GL_VERTEX_ARRAY);
	else if (enable & Attributes::Vertex)
		glEnableClientState(GL_VERTEX_ARRAY);

	if (disable & Attributes::Normal)
		glDisableClientState(GL_NORMAL_ARRAY);
	else if (enable & Attributes::Normal)
		glEnableClientState(GL_NORMAL_ARRAY);

	if (disable & Attributes::Colour)
		glDisableClientState(GL_COLOR_ARRAY);
	else if (enable & Attributes::Colour)
		glEnableClientState(GL_COLOR_ARRAY);

	if (disable & Attributes::Edge)
		glDisableClientState(GL_EDGE_FLAG_ARRAY);
	else if (enable & Attributes::Edge)
		glEnableClientState(GL_EDGE_FLAG_ARRAY);

	if (disable & Attributes::Fog)
		glDisableClientState(GL_FOG_COORD_ARRAY);
	else if (enable & Attributes::Fog)
		glEnableClientState(GL_FOG_COORD_ARRAY);

	if (disable & Attributes::Weights)
		glDisableClientState(GL_WEIGHT_ARRAY_ARB);
	else if (enable & Attributes::Weights)
		glEnableClientState(GL_WEIGHT_ARRAY_ARB);

	for (unsigned int i = 0; i < Texture::MaxTextures; i++) {
		if (disable & (Attributes::Texture0 << i)) {
			glClientActiveTextureARB(GL_TEXTURE0_ARB + i);
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		} else if (enable & (Attributes::Texture0 << i)) {
			glClientActiveTextureARB(GL_TEXTURE0_ARB + i);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		}
	}
}


}; };
