/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics3d.staticmodel.h"
#include "graphics3d.mesh.h"
#include "graphics3d.indexbuffer.h"
#include "graphics3d.vertexbuffer.h"
#include "graphics3d.material.h"
#include "graphics3d.shader.h"
#include "graphics3d.appearance.h"
#include "graphics3d.context.h"
#include "graphics3d.optimizer.h"

#include "base.string.inl"
#include "object.inl"
#include "object.pointer.inl"
#include "math.matrix.inl"
#include "math.vector.inl"
#include "container.iterator.inl"
#include "container.allocator.inl"
#include "container.tuple.inl"
#include "container.array.inl"
#include "system.program.inl"
#include "geometry.bounds.inl"
#include "geometry.sphere.inl"
#include "geometry.box.inl"
#include "geometry.alignedbox.inl"
#include "graphics3d.mesh.inl"
#include "graphics3d.model.inl"
#include "graphics3d.indexbuffer.inl"
#include "graphics3d.vertexbuffer.inl"
#include "graphics3d.vertexformat.inl"
#include "graphics3d.context.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.optimizer.inl"
#include "graphics3d.matrixstack.inl"
#include "graphics3d.appearance.inl"
#include "graphics3d.renderstream.inl"
#include "graphics3d.parameter.inl"
#include "graphics3d.parameterformat.inl"



namespace magnum { namespace graphics3d {
using namespace magnum::container;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StaticModel::clear() {
	m_Meshes.clear();
	clearParameters();
	clearJoints();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StaticModel::insertMesh(Mesh* m) {
	m_Meshes.insert(m);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StaticModel::removeMesh(Mesh* m) {
	for (unsigned int i = 0; i < m_Meshes.getSize(); i++) {
		if (m_Meshes[i] == m) {
			m_Meshes.remove(i);
			return;
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StaticModel::finish() {
	ParameterFormat fmt;
	fmt.insert(Parameters::Position, Types::Matrix4f, 0);
	clearParameters();
	publishParameters(fmt);
	
	clearJoints();

	Bounds3<float> bounds,tmpbnd;
	bounds.empty();
	Array<MeshP>::Iterator iter;
	for (iter = m_Meshes.begin(); iter != m_Meshes.end(); iter++) {
		if ((*iter)->bound(&tmpbnd))
			bounds.merge(tmpbnd);
	}
	publishBounds(bounds);		
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StaticModel::getMeshes(Array<Mesh*>* meshes) const {
	magnum::container::Array<MeshP>::Iterator iter, end;
	end = m_Meshes.end();

	for (iter = m_Meshes.begin(); iter != end; iter++) {
		meshes->insert(*iter);		
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StaticModel::onRender(const ParameterBuffer* pbuf) const {
	ASSERT(pbuf != NULL);
	MatrixStack* ms = getContext()->getMatrixStack();
	ms->selectMode(MatrixStack::ModelView);
	ms->set(*(const Matrix4<float>*)pbuf->getBuffer());

	if (isOptimized()) {
		m_Stream->execute();
	} else {
		magnum::container::Array<MeshP>::Iterator iter, end;
		end = m_Meshes.end();

		for (iter = m_Meshes.begin(); iter != end; iter++)
			(*iter)->render();
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool StaticModel::onOptimize() {
	Optimizer optimizer;

	magnum::container::Array<MeshP>::Iterator iter, end;
	end = m_Meshes.end();

	for (iter = m_Meshes.begin(); iter != end; iter++) {
		optimizer.renderMesh(*iter);
	}

	optimizer.finish();

	m_Stream = optimizer.optimize();
	m_Meshes.clear();

	return true;
}


}; };
