/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "object.h"
#include "base.string.h"
#include "graphics.image.h"
#include "gui.context.h"
#include "gui.bitmap.h"

#include "object.inl"
#include "base.string.inl"
#include "graphics.transform.inl"
#include "graphics.dimension.inl"
#include "graphics.image.inl"
#include "graphics.colour.inl"
#include "graphics.point.inl"
#include "graphics.rectangle.inl"
#include "gui.context.inl"
#include "gui.bitmap.inl"

#include <typeinfo.h>


#ifndef __FORWINDOWS__
#	error "This file is only for Windows"
#endif


namespace magnum { namespace gui { 

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context::Context() {
	m_DeviceHandle = NULL;
	m_WindowHandle = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context::~Context() {
	m_DeviceHandle = NULL;
	m_WindowHandle = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::setFillColour(const Colour<byte>& col) {
	m_FillColour = col;
	HBRUSH brush = CreateSolidBrush(RGB(col.R, col.G, col.B));
	HBRUSH oldBrush = (HBRUSH)SelectObject(m_DeviceHandle, brush);
	DeleteObject(oldBrush);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::setLineColour(const Colour<byte>& col) {
	m_LineColour = col;
	HPEN pen = CreatePen(PS_SOLID, m_LineWidth, RGB(col.R, col.G, col.B));
	HPEN oldPen = (HPEN)SelectObject(m_DeviceHandle, pen);
	DeleteObject(oldPen);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::setLineWidth(int w) {
	m_LineWidth = w;
	HPEN pen = CreatePen(PS_SOLID, m_LineWidth, RGB(m_LineColour.R, m_LineColour.G, m_LineColour.B));
	HPEN oldPen = (HPEN)SelectObject(m_DeviceHandle, pen);
	DeleteObject(oldPen);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Transform<float> Context::getTransform() const {
	XFORM worldTransform;
	Transform<float> t;

	// Save transformation matrix
	GetWorldTransform(m_DeviceHandle, &worldTransform);

	t.setTransformX(Point<float>(worldTransform.eM11, worldTransform.eM12));
	t.setTransformY(Point<float>(worldTransform.eM21, worldTransform.eM22));
	t.setTranslation(Point<float>(worldTransform.eDx, worldTransform.eDy));

	return t;
		
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::setTransform(const Transform<float>& tf)  {
	XFORM worldTransform;

	// Set new tranformation matrix
	worldTransform.eM11 = tf.getTransformX().getX();
	worldTransform.eM12 = tf.getTransformX().getY();
	worldTransform.eM21 = tf.getTransformY().getX();
	worldTransform.eM22 = tf.getTransformY().getY();
	worldTransform.eDx = tf.getTranslation().getX();
	worldTransform.eDy = tf.getTranslation().getY();
	SetWorldTransform(m_DeviceHandle, &worldTransform);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::setClipping(const Rectangle<float>& r) {

}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Rectangle<float> Context::getClipping() const {
	return Rectangle<float>();
}


}; };
