/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "GOpenGLView.h"
#include "GWindow.h"
#include "GChain.h"
#include "GDimension.h"
#include "GRectangle.h"
#include "GSystem.h"
#include "GMessage.h"

#include "GString.inl"
#include "GClass.inl"
#include "GObject.inl"
#include "GOpenGLView.inl"
#include "GView.inl"
#include "GChain.inl"
#include "GWindow.inl"
#include "GDimension.inl"
#include "GRectangle.inl"
#include "GPoint.inl"
#include "GIteratorBase.inl"

#include <GL/gl.h>

#ifndef __FORX11__
#	error "This file is only for X11"
#endif

namespace magnum { namespace gui {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
XVisualInfo* OpenGL::chooseFormat(Display* display) const {
	XVisualInfo *vi;
	int attribList[40];
	int *listPtr = attribList;

	// First assert TrueColour buffer
	*listPtr = GLX_RGBA; listPtr++;
	// We only want double-buffered visuals
	*listPtr = GLX_DOUBLEBUFFER; listPtr++;
	// Request colour deoth
	*listPtr = GLX_RED_SIZE; listPtr++;
		*listPtr = m_ColourBits / 3; listPtr++;
	*listPtr = GLX_GREEN_SIZE; listPtr++;
		*listPtr = m_ColourBits / 3; listPtr++;
	*listPtr = GLX_BLUE_SIZE; listPtr++;
		*listPtr = m_ColourBits / 3; listPtr++;
	// Request alpha depth (if needed)
	if (m_AlphaBits > 0) {
		*listPtr = GLX_ALPHA_SIZE; listPtr++;
			*listPtr = m_AlphaBits; listPtr++;
	}
	// Request z-depth (if needed)
	if (m_DepthBits > 0) {
		*listPtr = GLX_DEPTH_SIZE; listPtr++;
			*listPtr = m_DepthBits; listPtr++;
	}
	// Request accumulation buffer (if needed)
	if (m_AccumulationBits > 0) {
		int accum = m_AlphaBits > 0 ? m_AccumulationBits/3 : m_AccumulationBits/4;
		*listPtr = GLX_ACCUM_RED_SIZE; listPtr++;
			*listPtr = accum; listPtr++;
		*listPtr = GLX_ACCUM_GREEN_SIZE; listPtr++;
			*listPtr = accum; listPtr++;
		*listPtr = GLX_ACCUM_BLUE_SIZE; listPtr++;
			*listPtr = accum; listPtr++;
		if (m_AlphaBits > 0) {
			*listPtr = GLX_ACCUM_ALPHA_SIZE; listPtr++;
				*listPtr = accum; listPtr++;
		}	
	}
	// Request Stencil buffer (if needed)
	if (m_StencilBits > 0) {
		*listPtr = GLX_STENCIL_SIZE; listPtr++;
			*listPtr = m_StencilBits; listPtr++;
	}

	*listPtr = None;

	/* get an appropriate visual */
    vi = glXChooseVisual(display, DefaultScreen(display), attribList);
	return vi;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
XVisualInfo* OpenGL::adjustFormat(Display* display) {
	XVisualInfo* vi;
	int value;

	// Chose format
	vi = chooseFormat(display);

	// Return if no format found
	if (vi == NULL) {
		return NULL;
	}

	// Get all information from format
	glXGetConfig(display, vi, GLX_RED_SIZE, &value);
	m_ColourBits = value;
	glXGetConfig(display, vi, GLX_GREEN_SIZE, &value);
	m_ColourBits += value;
	glXGetConfig(display, vi, GLX_BLUE_SIZE, &value);
	m_ColourBits += value;
	glXGetConfig(display, vi, GLX_ALPHA_SIZE, &m_AlphaBits);
	glXGetConfig(display, vi, GLX_STENCIL_SIZE, &m_StencilBits);
	glXGetConfig(display, vi, GLX_DEPTH_SIZE, &m_DepthBits);
	glXGetConfig(display, vi, GLX_ACCUM_RED_SIZE, &value);
	m_AccumulationBits = value;
	glXGetConfig(display, vi, GLX_ACCUM_GREEN_SIZE, &value);
	m_AccumulationBits += value;
	glXGetConfig(display, vi, GLX_ACCUM_BLUE_SIZE, &value);
	m_AccumulationBits += value;
	glXGetConfig(display, vi, GLX_ACCUM_ALPHA_SIZE, &value);
	m_AccumulationBits += value;

	return vi;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGL::verifyFormat() const {
	Display* display;
	XVisualInfo* vi;

	if (m_Display == NULL) {
		display = XOpenDisplay(NULL);
	} else {
		display = m_Display;
	}

	vi = chooseFormat(display);

	if (m_Display == NULL) {
		XCloseDisplay(display);
	}

	if (vi == NULL) {
		return false;
	} else {
		XFree(vi);
		return true;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGL::adjustFormat() {
	Display* display;
	XVisualInfo* vi;
	int value;

	if (m_Display == NULL) {
		display = XOpenDisplay(NULL);
	} else {
		display = m_Display;
	}

	// Chose format
	vi = adjustFormat(display);

	// Return if no format found
	if (m_Display == NULL) {
		XCloseDisplay(display);
	}

	if (vi == NULL) {
		return false;
	} else {
		XFree(vi);
		return true;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGL::onAttachWindow() {
	XVisualInfo *vi;
	XSetWindowAttributes swa;

	GRectangle<int> frame = convertToWindow(getBounds());

	m_Display = getWindow()->getDisplayHandle();

	/* get an appropriate visual */
    vi = adjustFormat(m_Display);
    if (!vi) {
		m_Display = NULL;
		return false;
	}

    /* create a GLX context */
    m_OpenGLContext = glXCreateContext(m_Display, vi, 0, GL_TRUE);

    /* create a colormap */
    swa.colormap = XCreateColormap(m_Display, RootWindow(m_Display, vi->screen), vi->visual, AllocNone);

    /* create a window */
    swa.border_pixel = 0;
    swa.event_mask = 0; // ExposureMask;
	swa.do_not_propagate_mask = 0; // ExposureMask;
	swa.backing_store = NotUseful;
    m_WindowHandle = XCreateWindow(m_Display, getWindow()->getWindowHandle(), 
						frame.getX(), frame.getY(), frame.getWidth(), frame.getHeight(),
						0,
                        vi->depth, 
						InputOutput, 
						vi->visual,
                        CWBorderPixel|CWColormap|CWEventMask|CWDontPropagate|CWBackingStore, &swa);

	// Set window property...
	void* data = this;
	XChangeProperty(m_Display, m_WindowHandle, GLooper::g_ViewPointer, GLooper::g_IntegerType, 32, PropModeReplace, (unsigned char*)&data, sizeof(data)/4);

	// Show subwindow
	XMapWindow(m_Display, m_WindowHandle);

    /* connect the context to the window */
    glXMakeCurrent(m_Display, m_WindowHandle, m_OpenGLContext);
	XFree(vi);

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGL::onDetachWindow() {
	if (m_WindowHandle != 0) {
		glXDestroyContext(m_Display, m_OpenGLContext);
		XDestroyWindow(m_Display, m_WindowHandle);
		m_WindowHandle = 0;
		m_Display = NULL;
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGL::onResize(const GDimension<int>& size) {
	if (m_WindowHandle != 0) {
		GRectangle<int> bounds = convertToWindow(getBounds());
		XMoveResizeWindow(m_Display, m_WindowHandle, bounds.getX(), bounds.getY(), bounds.getWidth(), bounds.getHeight());
	    glXMakeCurrent(m_Display, m_WindowHandle, m_OpenGLContext);
		glViewport(0, 0, bounds.getWidth(), bounds.getHeight());
	}

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGL::onPaint(const GRectangle<int>& rect) {
	if (m_WindowHandle != 0) {
	    glXMakeCurrent(m_Display, m_WindowHandle, m_OpenGLContext);
		onRender();
		glXSwapBuffers(m_Display, m_WindowHandle);
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGL::dispatchEvent(const XEvent& event) {
	int len;

	switch(event.type) {
		case Expose:
GPRINT("OpenGL::dispatchEvent: Expose");
			// Collect some Expose events for performance reasons...
			len = event.xexpose.count;
			while ((len > 0) && (XPending(m_Display) > 0)) {
				XEvent event;
				XNextEvent(m_Display, &event);
				if ((event.type != Expose) || (event.xany.window != m_WindowHandle)) {
					XPutBackEvent(m_Display, &event);
					break;
				}
				len = event.xexpose.count;
			}
			onPaint(convertToWindow(getBounds()));
			return true;

		default:
			return true;
	}
}

}; };
