/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "gui3d.fontcache.h"
#include "gui3d.fontmap.h"

#include "gui3d.fontcache.inl"
#include "gui3d.fontmap.inl"
#include "object.pointer.inl"
#include "container.map.inl"
#include "container.allocator.inl"


namespace magnum { namespace gui3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
FontCache::~FontCache() {
	clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void FontCache::insert(const Font* font, FontMap* map) {
	ASSERT(map->getFont() == font);
	map->refer();
	m_Fonts.insert(font, map);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void FontCache::remove(const Font* font) {
	Iterator iter = m_Fonts.at(font);
	if (iter == m_Fonts.end())
		return;

	(*iter)->release();
	m_Fonts.remove(iter);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void FontCache::remove(const Iterator& iter) {
	(*iter)->release();
	m_Fonts.remove(iter);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void FontCache::clear() {
	Iterator iter;

	for (iter = m_Fonts.begin(); iter != m_Fonts.end(); ++iter) {
		(*iter)->release();
	}
	
	m_Fonts.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
FontMap* FontCache::get(const Font* font) {
	Iterator iter;
	
	iter = m_Fonts.at(font);
	if (iter != m_Fonts.end()) {
		return *iter;
	}
	
	// Create new fontmap
	FontMap* map = FontMap::create(font);
	m_Fonts.insert(font, map);
	
	return map;
}

}; };

