/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "gui3d.fontmap.h"

#include "gui3d.fontmap.inl"
#include "object.pointer.inl"
#include "freetype.font.inl"
#include "freetype.glyph.inl"
#include "graphics.point.inl"
#include "graphics.dimension.inl"
#include "graphics.rectangle.inl"


namespace magnum { namespace gui3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
FontMap::FontMap() {
	m_Bitmap = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
FontMap::~FontMap() {
	if (m_Bitmap != NULL) {
		m_Bitmap->release();
		m_Bitmap = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
FontMap* FontMap::create(const Font* font) {
	ASSERT(font != NULL);

	// Create Bitmap
	Bitmap* bmp = Bitmap::create(font->getBitmap()->getType(), font->getBitmap()->getSize(), 1);
	if (bmp == NULL)
		return NULL;
		
	// Copy image
	bmp->update(font->getBitmap(), 0);
		
	// Create FontMap object
	FontMap* fmap = new FontMap();
	fmap->m_Bitmap = bmp;
	fmap->m_Font = (Font*)font;
	
	// Create Glyph-Information
	float bwidth = (float)bmp->getWidth();
	float bheight = (float)bmp->getHeight();
	for (unsigned int i = 0; i < 256; i++) {
		fmap->m_Glyphs[i].width = (float)(*font)[i].getLocation().getWidth();
		fmap->m_Glyphs[i].depth = (float)(*font)[i].getLocation().getHeight() - (*font)[i].getBearing().getY();
		fmap->m_Glyphs[i].tx0 = (*font)[i].getLocation().getX() / bwidth;
		fmap->m_Glyphs[i].ty0 = (*font)[i].getLocation().getY() / bheight;
		fmap->m_Glyphs[i].tx1 = fmap->m_Glyphs[i].tx0 + (*font)[i].getLocation().getWidth() / bwidth;
		fmap->m_Glyphs[i].ty1 = fmap->m_Glyphs[i].ty0 + (*font)[i].getLocation().getHeight() / bheight;
		fmap->m_Glyphs[i].advance = (*font)[i].getSize().getWidth();
		fmap->m_Glyphs[i].bh = (*font)[i].getBearing().getX();
		fmap->m_Glyphs[i].bv = (*font)[i].getBearing().getY();
	}
	
	return fmap;
}

}; };

