/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "media.wavedevice.win.h"
#include "media.waveconsumer.win.h"
#include "media.waveconsumer.win.h"

#include "base.string.inl"
#include "object.class.inl"
#include "object.inl"
#include "system.thread.inl"
#include "system.signal.inl"
#include "container.queue.inl"
#include "media.node.inl"
#include "media.pin.inl"
#include "media.audio.inl"

#include "media.wavedevice.win.inl"
#include "media.waveconsumer.win.inl"

#ifndef __FORWINDOWS__
#	error "This file is only for Windows!"
#endif


namespace magnum { namespace media { namespace _windows {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WaveConsumer::WaveConsumer(WaveDevice* dev) : AudioConsumer(dev, Node::DefaultOutput) , m_InputPin(this) {
	m_Thread = NULL;
	publishInputPin(&m_InputPin);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WaveConsumer::~WaveConsumer() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveConsumer::onStart() {
	MMRESULT result;

	if (m_Thread != NULL)
		return false;

	m_Thread = new WaveThread(this);
	m_Thread->start();

	result = waveOutOpen(&m_WaveHandle, WAVE_MAPPER, &m_WaveFormat, m_Thread->getThreadId(), (DWORD)this, CALLBACK_THREAD | WAVE_FORMAT_DIRECT);
	if (result != MMSYSERR_NOERROR) {
		PostThreadMessage((DWORD)m_Thread->getThreadId(), WM_QUIT, NULL, NULL);
		m_Thread->join();
		m_Thread->release();
		m_Thread = NULL;
		return false;
	}

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveConsumer::onStop() {
	if (m_Thread == NULL)
		return false;

	waveOutReset(m_WaveHandle);
	waveOutClose(m_WaveHandle);
	PostThreadMessage((DWORD)m_Thread->getThreadId(), WM_QUIT, NULL, NULL);
	m_Thread->join();
	m_Thread->release();
	m_Thread = NULL;

	return true;	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveConsumer::onPause() {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveConsumer::onUnpause() {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveConsumer::onSetFormat(Pin* pin, const Format* fmt) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveConsumer::onAdjustFormat(Pin* pin, Format* fmt) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WaveConsumer::WaveThread::WaveThread(WaveConsumer *node) {
	m_Node = node;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WaveConsumer::WaveThread::~WaveThread() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool WaveConsumer::WaveThread::run() {
	MSG Message;
	WAVEHDR* hdr;
	Buffer* mbuf;
	PeekMessage(&Message, NULL, WM_USER, WM_USER, PM_NOREMOVE);

	while (::GetMessage(&Message, NULL, 0, 0)) {
		switch (Message.message) {
			case WOM_OPEN:
			case WOM_CLOSE:
				break;
			case WOM_DONE:
				hdr = (WAVEHDR *)Message.wParam;
				waveOutUnprepareHeader(m_Node->m_WaveHandle, hdr, sizeof(WAVEHDR));
				mbuf = (Buffer *)hdr->dwUser;
				mbuf->release();
				break;
			default:
				TranslateMessage(&Message);
				DispatchMessage(&Message);
		}
	}

	return true;
}

}; }; };
