/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "net.datagramsocket.h"
#include "net.exception.h"

#include "object.inl"
#include "base.string.inl"
#include "object.class.inl"
#include "object.exception.inl"
#include "net.socket.inl"
#include "net.datagramsocket.inl"

namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int DatagramSocket::receive(DatagramPacket &p) {
	unsigned long received, flags = 0;

#if defined(__FORWINDOWS__)
	if (WSARecv(m_Socket, &p.m_Buffer, 1, &received, &flags, NULL, NULL) != 0) {
		throwSocketException();
	}
#else
	received = recv(m_Socket, &p.m_Buffer, p.m_Len, flags);
	if (received == (unsigned long)-1) {
		throwSocketException();
	}
#endif

	return (int)received;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int DatagramSocket::send(DatagramPacket &p) {
	unsigned long sent;

#if defined(__FORWINDOWS__)
	if (WSASend(m_Socket, &p.m_Buffer, 1, &sent, 0, NULL, NULL) != 0) {
		throwSocketException();
	}
#else
	sent = sendto(m_Socket, &p.m_Buffer, p.m_Len, 0);
	if (sent == (unsigned long)-1) {
		throwSocketException();
	}
#endif

	return (int)sent;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int DatagramSocket::receiveFrom(DatagramPacket &p) {
	unsigned long received, flags = 0;
	sockaddr_in adr;
	int adrsize = sizeof(adr);

	adr.sin_family = AF_INET;
	adr.sin_port = p.m_Port;
	adr.sin_addr.s_addr = p.m_Address.getAddressNO();

#if defined(__FORWINDOWS__)
	if (WSARecvFrom(m_Socket, &p.m_Buffer, 1, &received, &flags, (sockaddr* )&adr, &adrsize, NULL, NULL) != 0) {
		throwSocketException();
	}
#else
	received = recvfrom(m_Socket, &p.m_Buffer, p.m_Len, 0, (sockaddr* )&adr, &adrsize);
	if (received == (unsigned long)-1) {
		throwSocketException();
	}
#endif

	p.m_Address.setAddressNO(adr.sin_addr.s_addr);
	p.m_Port = adr.sin_port;

	return (int)received;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int DatagramSocket::sendTo(DatagramPacket &p) {
	unsigned long sent;
	sockaddr_in adr;

	adr.sin_family = AF_INET;
	adr.sin_port = p.m_Port;
	adr.sin_addr.s_addr = p.m_Address.getAddressNO();

#if defined(__FORWINDOWS__)
	if (WSASendTo(m_Socket, &p.m_Buffer, 1, &sent, 0, (sockaddr* )&adr, sizeof(adr), NULL, NULL) != 0) {
		throwSocketException();
	}
#else
	sent = sendto(m_Socket, &p.m_Buffer, p.m_Len, 0, (sockaddr* )&adr, sizeof(adr));
	if (sent == (unsigned long)-1) {
		throwSocketException();
	}
#endif

	return (int)sent;
}
}; };
