/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "object.h"
#include "base.string.h"
#include "object.class.h"
#include "system.thread.h"
#include "system.mutex.h"

#include "object.inl"
#include "base.string.inl"
#include "object.class.inl"
#include "system.thread.inl"
#include "system.mutex.inl"


namespace magnum { namespace object {
using namespace magnum::system;

static Mutex	s_WaitingThreadsMutex;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Object::~Object() {
	m_Reference--;
	ASSERT(m_Reference == 0);

	while(m_WaitingThreads) {
		WaitingThread *thread = m_WaitingThreads;
		m_WaitingThreads = m_WaitingThreads->m_Next;
		delete thread;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Object::wait() {
	WaitingThread *thread = new WaitingThread;
	thread->m_Thread = Thread::getCurrent();
	s_WaitingThreadsMutex.lock();
	thread->m_Next = m_WaitingThreads;
	m_WaitingThreads = thread;
	s_WaitingThreadsMutex.unlock();
	thread->m_Thread->suspend();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Object::notify() {
	s_WaitingThreadsMutex.lock();
	WaitingThread *thread = m_WaitingThreads;
	if (thread != NULL) {
		m_WaitingThreads = thread->m_Next;
		thread->m_Thread->resume();
		delete thread;
	}
	s_WaitingThreadsMutex.unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Object::notifyAll() {
	WaitingThread *thread;
	s_WaitingThreadsMutex.lock();
	while (m_WaitingThreads != NULL) {
		thread = m_WaitingThreads;
		m_WaitingThreads = thread->m_Next;
		thread->m_Thread->resume();
		delete thread;
	}
	s_WaitingThreadsMutex.unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
String	Object::getName() const {
	return getClass().getName();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
String	Object::toString() const {
	return getClass().toString();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Object::isValid() const {
	return true;
}

}; };
