/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "object.property.h"

#include "object.inl"
#include "base.string.inl"
#include "container.array.inl"
#include "container.iterator.inl"
#include "object.property.inl"

namespace magnum { namespace object {



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Property::~Property() {
	if (m_Type == String) {
		if (m_Value.m_String != NULL)
			delete[] m_Value.m_String;
		if (m_Default.m_String != NULL) {
			if (m_Default.m_String != m_Value.m_String)
				delete[] m_Default.m_String;
		}
		m_Default.m_String =
		m_Value.m_String = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Property::setString(const char* val) {
	if (m_Value.m_String != NULL) {
		if (m_Value.m_String != m_Default.m_String)
			delete[] m_Value.m_String;
	}
	if (val == NULL) {
		m_Value.m_String = NULL;
		return;
	}
	size_t len = strlen(val);
	m_Value.m_String = new char[len+1];
	memcpy(m_Value.m_String, val, len+1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Property::setDefault() {
	if (m_Type == String) {
		if (m_Value.m_String == m_Default.m_String)
			return;
		if (m_Value.m_String != NULL)
			delete[] m_Value.m_String;
		m_Value.m_String = m_Default.m_String;
	} else {
		m_Value = m_Default;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
PropertyList::PropertyList() {

}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
PropertyList::PropertyList(const PropertyDescriptor* desc) {
	unsigned int size = 0, i = 0;

	while(desc[size].m_Type != Property::End)
		size++;

	m_Propertys.setSize(size);
	
	while(desc[i].m_Type != 0) {
		m_Propertys[i].m_Type = desc[i].m_Type;
		m_Propertys[i].m_Name = desc[i].m_Name;
		m_Propertys[i].m_Description = desc[i].m_Description;
		if (m_Propertys[i].m_Type != Property::String) {
			m_Propertys[i].m_Default = desc[i].m_Default;
		} else {
			size_t len = strlen(desc[i].m_Default.m_String);
			m_Propertys[i].m_Default.m_String = new char[len+1];
			memcpy(m_Propertys[i].m_Default.m_String, desc[i].m_Default.m_String, len+1);
		}
		m_Propertys[i].m_Value = m_Propertys[i].m_Default;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void PropertyList::append(const PropertyList* p) {
	unsigned int newsize, oldsize,i;

	oldsize = m_Propertys.getSize();
	newsize = p->m_Propertys.getSize() + oldsize;

	m_Propertys.setSize(newsize);
	for (i = oldsize; i < newsize; i++) {
		m_Propertys[i].m_Type = p->m_Propertys[i - oldsize].m_Type;
		m_Propertys[i].m_Name = p->m_Propertys[i - oldsize].m_Name;
		m_Propertys[i].m_Description = p->m_Propertys[i - oldsize].m_Description;
		if (m_Propertys[i].m_Type != Property::String) {
			m_Propertys[i].m_Default = p->m_Propertys[i - oldsize].m_Default;
			m_Propertys[i].m_Value = p->m_Propertys[i - oldsize].m_Value;
		} else {
			const char* str = p->m_Propertys[i - oldsize].m_Default.m_String;
			size_t len = strlen(str);
			m_Propertys[i].m_Default.m_String = new char[len+1];
			memcpy(m_Propertys[i].m_Default.m_String, str, len+1);
			if (p->m_Propertys[i - oldsize].m_Value.m_String == str) {
				m_Propertys[i].m_Value.m_String = m_Propertys[i].m_Default.m_String;
			} else {
				str = p->m_Propertys[i - oldsize].m_Value.m_String;
				len = strlen(str);
				m_Propertys[i].m_Value.m_String = new char[len+1];
				memcpy(m_Propertys[i].m_Value.m_String, str, len+1);
			}
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Property& PropertyList::operator[](const char* name) {
	Iterator iter = m_Propertys.begin();
	Iterator end = m_Propertys.end();

	while (iter != end) {
		if (iter->m_Name == name)
			return *iter;
		iter++;
	}

	ASSERT(false);
	return *(Property*)(void*)NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
const Property& PropertyList::operator[](const char* name) const {
	Iterator iter = m_Propertys.begin();
	Iterator end = m_Propertys.end();

	while (iter != end) {
		if (iter->m_Name == name)
			return *iter;
		iter++;
	}

	ASSERT(false);
	return *(Property*)(void*)NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
PropertyList::Iterator PropertyList::at(const char* name) const {
	Iterator iter = m_Propertys.begin();
	Iterator end = m_Propertys.end();

	while (iter != end) {
		if (iter->m_Name == name)
			return iter;
		iter++;
	}

	return end;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool PropertyList::contains(const char* name) const {
	Iterator iter = m_Propertys.begin();
	Iterator end = m_Propertys.end();

	while (iter != end) {
		if (iter->m_Name == name)
			return true;
		iter++;
	}

	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void PropertyList::setDefault() {
	Iterator iter = m_Propertys.begin();
	Iterator end = m_Propertys.end();

	while (iter != end) {
		iter->setDefault();
		iter++;
	}
}

}; };
