/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "scene3d.modelnode.h"
#include "scene3d.scheduler.h"
#include "scene3d.locale.h"
#include "scene3d.audio.h"
#include "scene3d.behaviour.h"
#include "graphics3d.parameter.h"
#include "graphics3d.types.h"

#include "object.pointer.inl"
#include "container.iterator.inl"
#include "math.vector.inl"
#include "math.matrix.inl"
#include "geometry.space.inl"
#include "geometry.sphere.inl"
#include "geometry.bounds.inl"
#include "geometry.transform.inl"
#include "graphics3d.parameter.inl"
#include "graphics3d.parameterformat.inl"
#include "graphics3d.model.inl"
#include "scene3d.modelnode.inl"
#include "scene3d.scheduler.inl"
#include "scene3d.leaf.inl"
#include "scene3d.node.inl"
#include "scene3d.geometry.inl"


namespace magnum {namespace scene3d {
using magnum::graphics3d::Parameters;
using magnum::graphics3d::ParameterBuffer;
using magnum::graphics3d::ParameterFormat;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ModelNode::onSchedule(RenderQueue* rq, const Transform3<float>& camera) {
	rq->insert(this);
	*m_ModelMatrix = (Matrix4<float>)(camera * getWorldTransform());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ModelNode::attachModel(Model* model) {
	ASSERT(model != NULL);
	
	// Initialize some member vars
	if (m_Parameters != NULL)
		m_Parameters->release();
	m_Parameters = NULL;
	m_ModelMatrix = NULL;
	m_Model = model;

	// Try to get the parameter format
	const ParameterFormat* fmt = &m_Model->getParameters();
	if (fmt == NULL)
		return;

	// Create a parameter buffer
	m_Parameters = ParameterBuffer::create(*fmt);

	// Get Pointer to transformation matrix
	ParameterFormat::Iterator iter = fmt->at(Parameters::Position);
	if (iter != fmt->end())
		m_ModelMatrix = (Matrix4<float>*)m_Parameters->getPointer(iter->getLocation());
		
	// Get bounding volume
	setBounds(m_Model->getBounds());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ModelNode::detachModel() {
	if (m_Parameters != NULL)
		m_Parameters->release();
	m_Parameters = NULL;
	m_Model = NULL;
	m_ModelMatrix = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ModelNode::onRender() const {
	ASSERT(m_Model != NULL);
	ASSERT(m_ModelMatrix != NULL);

	m_Model->render(m_Parameters);
}


}; };
