/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "scene3d.scheduler.h"
#include "scene3d.universe.h"
#include "scene3d.observer.h"
#include "scene3d.activatornode.h"

#include "object.pointer.inl"
#include "container.scheduler.inl"
#include "geometry.space.inl"
#include "scene3d.universe.inl"
#include "scene3d.scheduler.inl"
#include "scene3d.traverser.inl"
#include "scene3d.timer.inl"

namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
BehaviourScheduler::BehaviourScheduler(Observer* o) {
	ASSERT(o != NULL);
	m_Observer = o;
	m_Universe = o->getUniverse();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
BehaviourScheduler::~BehaviourScheduler() {
	clear();
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BehaviourScheduler::process() {
	// Process all behaviour nodes
	magnum::container::Scheduler<Behaviour>::Iterator iter;
	Timer* timer = m_Universe->getTimer();
	iter = m_ActiveBehaviours.begin();
	while(iter != m_ActiveBehaviours.end()) {
		Behaviour* b = (Behaviour*)iter;
		iter++;

		// Is this behaviour node still active?
		if (b->testFlag(Node::ActiveBehaviour)) {
			b->clearFlag(Node::ActiveBehaviour);
			b->onProcess(*timer);
		} else {
			m_ActiveBehaviours.remove(b);
		}
	}

	// Update dirty nodes
	m_Universe->flushDirtyNodes();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BehaviourScheduler::clear() {
	while(!m_ActiveBehaviours.isEmpty()) {
		Behaviour* b = (Behaviour*)m_ActiveBehaviours.begin();
		b->clearFlag(Node::ActiveBehaviour);
		m_ActiveBehaviours.remove(b);
	}
}


}; };
